## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
## 
## CvEventManager
## This class is passed an argsList from CvAppInterface.onEvent
## The argsList can contain anything from mouse location to key info
## The EVENTLIST that are being notified can be found 


from CvPythonExtensions import *
import CvUtil
import CvScreensInterface
import CvDebugTools
import CvWBPopups
import PyHelpers
import Popup as PyPopup
import CvCameraControls
import CvTopCivs
import sys
import CvWorldBuilderScreen
import CvAdvisorUtils
import CvTechChooser
import GodsOfOld
import pickle

gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame


# globals
###################################################


g_iNumNukesWarningMessage = 25
g_iNumNukesGameOver = 50



class CvEventManager:
	def __init__(self):
		#################### ON EVENT MAP ######################
		# Gods variables
		# private
		self.clearGodsVariables()
				
		#print "EVENTMANAGER INIT"
		
		self.bCtrl = False
		self.bShift = False
		self.bAlt = False
		self.bAllowCheats = False
		
		# OnEvent Enums
		self.EventLButtonDown=1
		self.EventLcButtonDblClick=2
		self.EventRButtonDown=3
		self.EventBack=4
		self.EventForward=5
		self.EventKeyDown=6
		self.EventKeyUp=7
	
		self.__LOG_MOVEMENT = 0
		self.__LOG_BUILDING = 0
		self.__LOG_COMBAT = 0
		self.__LOG_CONTACT = 0
		self.__LOG_IMPROVEMENT =0
		self.__LOG_CITYLOST = 0
		self.__LOG_CITYBUILDING = 0
		self.__LOG_TECH = 0
		self.__LOG_UNITBUILD = 0
		self.__LOG_UNITKILLED = 1
		self.__LOG_UNITLOST = 0
		self.__LOG_UNITPROMOTED = 0
		self.__LOG_UNITSELECTED = 0
		self.__LOG_UNITPILLAGE = 0
		self.__LOG_GOODYRECEIVED = 0
		self.__LOG_GREATPERSON = 0
		self.__LOG_RELIGION = 0
		self.__LOG_RELIGIONSPREAD = 0
		self.__LOG_GOLDENAGE = 0
		self.__LOG_ENDGOLDENAGE = 0
		self.__LOG_WARPEACE = 0
		self.__LOG_PUSH_MISSION = 0
		self.DemoCount = 0
		
		#Net Messages
		self.m_iNetMessage_Inquisitor = 0
		self.m_iNetMessage_ProphetKi = 1
		self.m_iNetMessage_ProphetEnki = 2
		self.m_iNetMessage_ProphetEnlil = 3
		self.m_iNetMessage_ProphetNanna = 4
		self.m_iNetMessage_ProphetUtu = 5
		self.m_iNetMessage_ProphetAn = 6
		
		# Next War tracks cities that have been razed
		self.iArcologyCityID = -1
		
		## EVENTLIST
		self.EventHandlerMap = {
			'mouseEvent'			: self.onMouseEvent,
			'kbdEvent' 				: self.onKbdEvent,
			'ModNetMessage'					: self.onModNetMessage,
			'Init'					: self.onInit,
			'Update'				: self.onUpdate,
			'UnInit'				: self.onUnInit,
			'OnSave'				: self.onSaveGame,
			'OnPreSave'				: self.onPreSave,
			'OnLoad'				: self.onLoadGame,
			'GameStart'				: self.onGameStart,
			'GameEnd'				: self.onGameEnd,
			'plotRevealed' 			: self.onPlotRevealed,
			'plotFeatureRemoved' 	: self.onPlotFeatureRemoved,
			'plotPicked'			: self.onPlotPicked,
			'nukeExplosion'			: self.onNukeExplosion,
			'gotoPlotSet'			: self.onGotoPlotSet,
			'BeginGameTurn'			: self.onBeginGameTurn,
			'EndGameTurn'			: self.onEndGameTurn,
			'BeginPlayerTurn'		: self.onBeginPlayerTurn,
			'EndPlayerTurn'			: self.onEndPlayerTurn,
			'endTurnReady'			: self.onEndTurnReady,
			'combatResult' 			: self.onCombatResult,
		  'combatLogCalc'	 		: self.onCombatLogCalc,
		  'combatLogHit'				: self.onCombatLogHit,
			'improvementBuilt' 		: self.onImprovementBuilt,
			'improvementDestroyed' 		: self.onImprovementDestroyed,
			'routeBuilt' 		: self.onRouteBuilt,
			'firstContact' 			: self.onFirstContact,
			'cityBuilt' 			: self.onCityBuilt,
			'cityRazed'				: self.onCityRazed,
			'cityAcquired' 			: self.onCityAcquired,
			'cityAcquiredAndKept' 	: self.onCityAcquiredAndKept,
			'cityLost'				: self.onCityLost,
			'cultureExpansion' 		: self.onCultureExpansion,
			'cityGrowth' 			: self.onCityGrowth,
			'cityDoTurn' 			: self.onCityDoTurn,
			'cityBuildingUnit'	: self.onCityBuildingUnit,
			'cityBuildingBuilding'	: self.onCityBuildingBuilding,
			'cityRename'				: self.onCityRename,
			'cityHurry'				: self.onCityHurry,
			'selectionGroupPushMission'		: self.onSelectionGroupPushMission,
			'unitMove' 				: self.onUnitMove,
			'unitSetXY' 			: self.onUnitSetXY,
			'unitCreated' 			: self.onUnitCreated,
			'unitBuilt' 			: self.onUnitBuilt,
			'unitKilled'			: self.onUnitKilled,
			'unitLost'				: self.onUnitLost,
			'unitPromoted'			: self.onUnitPromoted,
			'unitSelected'			: self.onUnitSelected, 
			'UnitRename'				: self.onUnitRename,
			'unitPillage'				: self.onUnitPillage,
			'unitSpreadReligionAttempt'	: self.onUnitSpreadReligionAttempt,
			'unitGifted'				: self.onUnitGifted,
			'unitBuildImprovement'				: self.onUnitBuildImprovement,
			'goodyReceived'        	: self.onGoodyReceived,
			'greatPersonBorn'      	: self.onGreatPersonBorn,
			'buildingBuilt' 		: self.onBuildingBuilt,
			'projectBuilt' 			: self.onProjectBuilt,
			'techAcquired'			: self.onTechAcquired,
			'techSelected'			: self.onTechSelected,
			'religionFounded'		: self.onReligionFounded,
			'religionSpread'		: self.onReligionSpread, 
			'religionRemove'		: self.onReligionRemove, 
			'corporationFounded'	: self.onCorporationFounded,
			'corporationSpread'		: self.onCorporationSpread, 
			'corporationRemove'		: self.onCorporationRemove, 
			'goldenAge'				: self.onGoldenAge,
			'endGoldenAge'			: self.onEndGoldenAge,
			'chat' 					: self.onChat,
			'victory'				: self.onVictory,
			'vassalState'			: self.onVassalState,
			'changeWar'				: self.onChangeWar,
			'setPlayerAlive'		: self.onSetPlayerAlive,
			'playerChangeStateReligion'		: self.onPlayerChangeStateReligion,
			'playerGoldTrade'		: self.onPlayerGoldTrade,
			'windowActivation'		: self.onWindowActivation,
			'gameUpdate'			: self.onGameUpdate,		# sample generic event
		}

		################## Events List ###############################
		#
		# Dictionary of Events, indexed by EventID (also used at popup context id)
		#   entries have name, beginFunction, applyFunction [, randomization weight...]
		#
		# Normal events first, random events after
		#	
		################## Events List ###############################
		self.Events={
			CvUtil.EventEditCityName : ('EditCityName', self.__eventEditCityNameApply, self.__eventEditCityNameBegin),
			CvUtil.EventEditCity : ('EditCity', self.__eventEditCityApply, self.__eventEditCityBegin),
			CvUtil.EventPlaceObject : ('PlaceObject', self.__eventPlaceObjectApply, self.__eventPlaceObjectBegin),
			CvUtil.EventAwardTechsAndGold: ('AwardTechsAndGold', self.__eventAwardTechsAndGoldApply, self.__eventAwardTechsAndGoldBegin),
			CvUtil.EventEditUnitName : ('EditUnitName', self.__eventEditUnitNameApply, self.__eventEditUnitNameBegin),
			CvUtil.EventWBAllPlotsPopup : ('WBAllPlotsPopup', self.__eventWBAllPlotsPopupApply, self.__eventWBAllPlotsPopupBegin),
			CvUtil.EventWBLandmarkPopup : ('WBLandmarkPopup', self.__eventWBLandmarkPopupApply, self.__eventWBLandmarkPopupBegin),
			CvUtil.EventWBScriptPopup : ('WBScriptPopup', self.__eventWBScriptPopupApply, self.__eventWBScriptPopupBegin),
			CvUtil.EventWBStartYearPopup : ('WBStartYearPopup', self.__eventWBStartYearPopupApply, self.__eventWBStartYearPopupBegin),
		}	
#################### EVENT STARTERS ######################
	def handleEvent(self, argsList):
		'EventMgr entry point'
		# extract the last 6 args in the list, the first arg has already been consumed
		self.origArgsList = argsList	# point to original
		tag = argsList[0]				# event type string
		idx = len(argsList)-6
		bDummy = false
		self.bDbg, bDummy, self.bAlt, self.bCtrl, self.bShift, self.bAllowCheats = argsList[idx:]
		ret = 0
		if self.EventHandlerMap.has_key(tag):
			fxn = self.EventHandlerMap[tag]
			ret = fxn(argsList[1:idx])
		return ret
		
#################### EVENT APPLY ######################	
	def beginEvent( self, context, argsList=-1 ):
		'Begin Event'
		entry = self.Events[context]
		return entry[2]( argsList )
	
	def applyEvent( self, argsList ):
		'Apply the effects of an event '
		context, playerID, netUserData, popupReturn = argsList
		
		if context == CvUtil.PopupTypeEffectViewer:
			return CvDebugTools.g_CvDebugTools.applyEffectViewer( playerID, netUserData, popupReturn )
		
		entry = self.Events[context]
				
		if ( context not in CvUtil.SilentEvents ):
			self.reportEvent(entry, context, (playerID, netUserData, popupReturn) )
		return entry[1]( playerID, netUserData, popupReturn )   # the apply function

	def reportEvent(self, entry, context, argsList):
		'Report an Event to Events.log '
		if (gc.getGame().getActivePlayer() != -1):
			message = "DEBUG Event: %s (%s)" %(entry[0], gc.getActivePlayer().getName())
			CyInterface().addImmediateMessage(message,"")
			CvUtil.pyPrint(message)
		return 0
		
#################### ON EVENTS ######################
	def onKbdEvent(self, argsList):
		'keypress handler - return 1 if the event was consumed'

		eventType,key,mx,my,px,py = argsList
		game = CyGame()
		
		if (self.bAllowCheats):
			# notify debug tools of input to allow it to override the control
			argsList = (eventType,key,self.bCtrl,self.bShift,self.bAlt,mx,my,px,py,CyGame().isNetworkMultiPlayer())
			if ( CvDebugTools.g_CvDebugTools.notifyInput(argsList) ):
				return 0
		
		if ( eventType == self.EventKeyDown ):
			theKey=int(key)
			
			CvCameraControls.g_CameraControls.handleInput( theKey )
						
			# Shift - T (Debug - No MP)
			if (self.bAllowCheats and theKey == int(InputTypes.KB_T)):
				if ( self.bShift ):
					self.beginEvent(CvUtil.EventAwardTechsAndGold)
					#self.beginEvent(CvUtil.EventCameraControlPopup)
					return 1
						
			# Shift - ] (Debug - currently mouse-overd unit, health += 10
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_LBRACKET) and self.bShift ):
				unit = CyMap().plot(px, py).getUnit(0)
				if ( not unit.isNone() ):
					d = min( unit.maxHitPoints()-1, unit.getDamage() + 10 )
					unit.setDamage( d, PlayerTypes.NO_PLAYER )
				
			# Shift - [ (Debug - currently mouse-overd unit, health -= 10
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_RBRACKET) and self.bShift ):
				unit = CyMap().plot(px, py).getUnit(0)
				if ( not unit.isNone() ):
					d = max( 0, unit.getDamage() - 10 )
					unit.setDamage( d, PlayerTypes.NO_PLAYER )
				
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_F1)):
				if ( self.bShift ):
					CvScreensInterface.replayScreen.showScreen(False)
					return 1
				# don't return 1 unless you want the input consumed
			
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_F2)):
				if ( self.bShift ):
					import CvDebugInfoScreen
 					CvScreensInterface.showDebugInfoScreen()
					return 1
			
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_F3)):
				if ( self.bShift ):
					CvScreensInterface.showDanQuayleScreen(())
					return 1
					
			elif (self.bAllowCheats and theKey == int(InputTypes.KB_F4)):
				if ( self.bShift ):
					CvScreensInterface.showUnVictoryScreen(())
					return 1
					
		return 0

	def onModNetMessage(self, argsList):
		'Called whenever CyMessageControl().sendModNetMessage() is called - this is all for you modders!'
		
		iData1, iData2, iData3, iData4, iData5 = argsList
		
		print("Modder's net message!")
		
		CvUtil.pyPrint( 'onModNetMessage' )

		iMessageID = iData1
		
#Inquisitor's Effect on a City
		if ( iMessageID == self.m_iNetMessage_Inquisitor ):
			
			iPlotX = iData2
			iPlotY = iData3
			iOwner = iData4
			iUnitID = iData5
			
			pPlot = CyMap( ).plot( iPlotX, iPlotY )
			pCity = pPlot.getPlotCity( )
			pPlayer = gc.getPlayer( iOwner )
			pUnit = pPlayer.getUnit( iUnitID )
			
			self.doInquisitorPersecution( pCity, pUnit )
			
# Social Reform
		if ( iMessageID == self.m_iNetMessage_ProphetKi ):
			
			iPlotX = iData2
			iPlotY = iData3
			iOwner = iData4
			iUnitID = iData5
			
			pPlot = CyMap( ).plot( iPlotX, iPlotY )					
			pPlayer = gc.getPlayer( iOwner )
			pUnit = pPlayer.getUnit( iUnitID )

			self.doEventEarthquake(pPlot)
			pUnit.kill( 0, -1 )
			if pPlot.getOwner( ) >= 0:		
				if not ( gc.getPlayer( pPlot.getOwner( ) ).isHuman( ) ):
					gc.getPlayer( pPlot.getOwner()).AI_changeAttitudeExtra( iOwner, 0 )
					szAttackedCiv = gc.getPlayer( pPlot.getOwner()).getCivilizationDescriptionKey()
					szAttackingCiv = pPlayer.getCivilizationDescriptionKey()
					szTitle = localText.getText("TXT_KEY_SOCIAL_REFORM", (szAttackedCiv , szAttackingCiv, ))
					CyInterface().addImmediateMessage( szTitle , None)
				else:
					szAttackedCiv = gc.getPlayer( pPlot.getOwner()).getCivilizationDescriptionKey()
					szAttackingCiv = pPlayer.getCivilizationDescriptionKey()
					szTitle = localText.getText("TXT_KEY_SOCIAL_REFORM", (szAttackedCiv , szAttackingCiv, ))				
	
#Lawyer's Effect on a City
		if ( iMessageID == self.m_iNetMessage_ProphetEnki ):
			
			iPlotX = iData2
			iPlotY = iData3
			iOwner = iData4
			iUnitID = iData5
			
			pPlot = CyMap( ).plot( iPlotX, iPlotY )
			pCity = pPlot.getPlotCity( )
			pPlayer = gc.getPlayer( iOwner )
			pUnit = pPlayer.getUnit( iUnitID )
			
			self.doEventLawyer( pCity, pUnit )


# Bubonic Plague
		if ( iMessageID == self.m_iNetMessage_ProphetEnlil ):

			iPlotX = iData2
			iPlotY = iData3
			iOwner = iData4
			iUnitID = iData5
			
			pPlot = CyMap( ).plot( iPlotX, iPlotY )					
			pPlayer = gc.getPlayer( iOwner )
			pUnit = pPlayer.getUnit( iUnitID )			
			
			self.doEventPlague(pPlot)
			pUnit.kill( 0, -1 )
			if pPlot.getOwner( ) >= 0:
				if not ( gc.getPlayer( pPlot.getOwner( ) ).isHuman( ) ):
					gc.getPlayer( pPlot.getOwner()).AI_changeAttitudeExtra( iOwner, -3 )
					szAttackedCiv = gc.getPlayer( pPlot.getOwner()).getCivilizationDescriptionKey()
					szAttackingCiv = pPlayer.getCivilizationDescriptionKey()
					szTitle = localText.getText("TXT_KEY_GODS_DISASTER_ANGER", (szAttackedCiv , szAttackingCiv, ))
					CyInterface().addImmediateMessage( szTitle , None)
				else:
					szAttackedCiv = gc.getPlayer( pPlot.getOwner()).getCivilizationDescriptionKey()
					szAttackingCiv = pPlayer.getCivilizationDescriptionKey()
					szTitle = localText.getText("TXT_KEY_GODS_DISASTER_HUMAN", (szAttackedCiv , szAttackingCiv, ))				


# Diplomacy
		if ( iMessageID == self.m_iNetMessage_ProphetNanna ):

			iPlotX = iData2
			iPlotY = iData3
			iOwner = iData4
			iUnitID = iData5
			
			pPlot = CyMap( ).plot( iPlotX, iPlotY )					
			pPlayer = gc.getPlayer( iOwner )
			pUnit = pPlayer.getUnit( iUnitID )			
			
			pUnit.kill( 0, -1 )
			self.doEventMinorMeteorStrike(pPlot)
			if pPlot.getOwner( ) >= 0:

				if not ( gc.getPlayer( pPlot.getOwner( ) ).isHuman( ) ):
					gc.getPlayer( pPlot.getOwner()).AI_changeAttitudeExtra( iOwner, +3 )
					szAttackedCiv = gc.getPlayer( pPlot.getOwner()).getCivilizationDescriptionKey()
					szAttackingCiv = pPlayer.getCivilizationDescriptionKey()
					szTitle = localText.getText("TXT_KEY_GODS_DIPLOMAT_ANGER", (szAttackedCiv , szAttackingCiv, ))
					CyInterface().addImmediateMessage( szTitle , None)
				else:
					szAttackedCiv = gc.getPlayer( pPlot.getOwner()).getCivilizationDescriptionKey()
					szAttackingCiv = pPlayer.getCivilizationDescriptionKey()
					szTitle = localText.getText("TXT_KEY_GODS_DIPLOMAT_HUMAN", (szAttackedCiv , szAttackingCiv, ))				


# Super Virus
		if ( iMessageID == self.m_iNetMessage_ProphetUtu ):

			iPlotX = iData2
			iPlotY = iData3
			iOwner = iData4
			iUnitID = iData5
			
			pPlot = CyMap( ).plot( iPlotX, iPlotY )					
			pPlayer = gc.getPlayer( iOwner )
			pUnit = pPlayer.getUnit( iUnitID )			
			
			self.doEventBlight(pPlot)
			pUnit.kill( 0, -1 )
			if pPlot.getOwner( ) >= 0:
				if not ( gc.getPlayer( pPlot.getOwner( ) ).isHuman( ) ):
					gc.getPlayer( pPlot.getOwner()).AI_changeAttitudeExtra( iOwner, -4 )
					szAttackedCiv = gc.getPlayer( pPlot.getOwner()).getCivilizationDescriptionKey()
					szAttackingCiv = pPlayer.getCivilizationDescriptionKey()
					szTitle = localText.getText("TXT_KEY_GODS_DISASTER_ANGER", (szAttackedCiv , szAttackingCiv, ))
					CyInterface().addImmediateMessage( szTitle , None)
				else:
					szAttackedCiv = gc.getPlayer( pPlot.getOwner()).getCivilizationDescriptionKey()
					szAttackingCiv = pPlayer.getCivilizationDescriptionKey()
					szTitle = localText.getText("TXT_KEY_GODS_DISASTER_HUMAN", (szAttackedCiv , szAttackingCiv, ))				

			
# Diplomatic Immunity
		if ( iMessageID == self.m_iNetMessage_ProphetAn ):

			iAttackingOwner = iData2
			iAttackingUnitID = iData3
			iDefendingOwner = iData4
			iDefendingUnitID = iData5
			
			pAttackingPlayer = gc.getPlayer( iAttackingOwner )
			pAttackingUnit = pAttackingPlayer.getUnit( iAttackingUnitID )			
			pDefendingPlayer = gc.getPlayer( iDefendingOwner )
			pDefendingUnit = pDefendingPlayer.getUnit( iDefendingUnitID )

			pAttackingUnit.kill( 0, -1 )			
			pDefendingUnit.kill( 0, -1 )

			if not ( pDefendingPlayer.isHuman( ) ):
				pDefendingPlayer.AI_changeAttitudeExtra( iAttackingOwner, -1 )

				szAttackedCiv = pDefendingPlayer.getCivilizationDescriptionKey()
				szAttackingCiv = pAttackingPlayer.getCivilizationDescriptionKey()
				szTitle = localText.getText("TXT_KEY_GODS_DISASTER_ANGER_TWO1", (szAttackedCiv , szAttackingCiv, ))
				CyInterface().addImmediateMessage( szTitle , None)			
			else:
				szAttackedCiv = pDefendingPlayer.getCivilizationDescriptionKey()
				szAttackingCiv = pAttackingPlayer.getCivilizationDescriptionKey()
				szTitle = localText.getText("TXT_KEY_GODS_DISASTER_HUMAN_TWO1", (szAttackedCiv , szAttackingCiv, ))
				CyInterface().addImmediateMessage( szTitle , None)	
			
						
	def onInit(self, argsList):
		'Called when Civ starts up'
		CvUtil.pyPrint( 'OnInit' )
		
	def onUpdate(self, argsList):
		'Called every frame'
		fDeltaTime = argsList[0]

		# allow camera to be updated
		CvCameraControls.g_CameraControls.onUpdate( fDeltaTime )
		
	def onWindowActivation(self, argsList):
		'Called when the game window activates or deactivates'
		bActive = argsList[0]
		
	def onUnInit(self, argsList):
		'Called when Civ shuts down'
		CvUtil.pyPrint('OnUnInit')
	
	def onPreSave(self, argsList):
		"called before a game is actually saved"
		CvUtil.pyPrint('OnPreSave')
		self.setGodsVariables()
	
	def onSaveGame(self, argsList):
		"return the string to be saved - Must be a string"
		return ""

	def onLoadGame(self, argsList):
		CvAdvisorUtils.resetNoLiberateCities()
		self.getGodsVariables()
		return 0

	def onGameStart(self, argsList):
		'Called at the start of the game'
		
		self.clearGodsVariables( )
		
		self.initScriptData()
		
		# Are we using the scenario file? If so, then show the backstory popup
		if (CyMap().plot(0,0).getScriptData() == "Scenario"):
			for iPlayer in range(gc.getMAX_PLAYERS()):
				player = gc.getPlayer(iPlayer)
				if (player.isAlive() and player.isHuman()):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_TEXT)
					szTitle = u"<font=4b>" + localText.getText("TXT_KEY_NEXT_WAR_BACKSTORY_TITLE", ()) + u"</font>"
					szBody = u"<font=3>" + localText.getText("TXT_KEY_NEXT_WAR_BACKSTORY_TEXT", ()) + u"</font>"
					popupInfo.setText(szTitle + u"\n\n" + szBody)
					popupInfo.addPopup(iPlayer)
		
		if (gc.getGame().getStartEra() == gc.getDefineINT("STANDARD_ERA")):
			for iPlayer in range(gc.getMAX_PLAYERS()):
				player = gc.getPlayer(iPlayer)
				if (player.isAlive() and player.isHuman()):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
					popupInfo.setText(u"showDawnOfMan")
					popupInfo.addPopup(iPlayer)
		else:
			CyInterface().setSoundSelectionReady(true)

		if gc.getGame().isPbem():
			for iPlayer in range(gc.getMAX_PLAYERS()):
				player = gc.getPlayer(iPlayer)
				if (player.isAlive() and player.isHuman()):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_DETAILS)
					popupInfo.setOption1(true)
					popupInfo.addPopup(iPlayer)

## Str Trait Start ##

		for iPlayerLoop in range(gc.getMAX_CIV_PLAYERS()):

			pPlayer = gc.getPlayer(iPlayerLoop)

			if (pPlayer.isAlive()):

				iTrait = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_STRATEGIC')
			
				if (pPlayer.hasTrait(iTrait)):

					estiEnd = CyGame().getEstimateEndTurn()
					if ( estiEnd >= 1000 ):
						pPlayer.setGold(300)
					elif ( estiEnd >= 700 ):
						pPlayer.setGold(250)
					elif ( estiEnd >= 500 ):
						pPlayer.setGold(200)
					elif ( estiEnd >= 300 ):
						pPlayer.setGold(150)

## Str Trait End ##

		CvAdvisorUtils.resetNoLiberateCities()					
		
	def onGameEnd(self, argsList):
		'Called at the End of the game'
		print("Game is ending")
		return

	def onBeginGameTurn(self, argsList):
		'Called at the beginning of the end of each turn'
		iGameTurn = argsList[0]
		CvTopCivs.CvTopCivs().turnChecker(iGameTurn)

#Plague
		if gc.getMap().plotByIndex(0).getScriptData() == "Plague On":
			self.doPlagueSpread()
		else:
			self.doPlagueDecay()

#Constalations
		if ( iGameTurn % 10 ) == 0:
			self.doChooseNewStarAlignment( )

#Next War
		if gc.getMap().getMapScriptName() == "NextWar.CivBeyondSwordWBSave":
			if iGameTurn != 0:
				if iGameTurn%2 == 0:
					self.doCheckDepletion()
#Next War


	def onEndGameTurn(self, argsList):
		'Called at the end of the end of each turn'
		iGameTurn = argsList[0]
		
	def onBeginPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList

		if not gc.getPlayer( iPlayer ).isHuman():
			self.doAIOperations( iPlayer )

## e-Bank and Suzerainty Start ##

		pPlayer = gc.getPlayer(iPlayer)
		b_eBank = gc.getInfoTypeForString("BUILDING_E_BANK")
		obsoleteTech = gc.getBuildingInfo(b_eBank).getObsoleteTech()

		if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
			for iCity in range(pPlayer.getNumCities()):
				ppCity = pPlayer.getCity(iCity)
				if ppCity.getNumActiveBuilding(b_eBank) == true:
					pPID = pPlayer.getID()
					iGold = pPlayer.getGold( )
					if pPlayer.getGold( ) >= 2500:
						pPlayer.changeGold( 50 )
					if pPlayer.getGold( ) < 2500:
						if pPlayer.getGold( ) >= 100:
							pPlayer.changeGold( iGold//50 )
						else:
							pPlayer.changeGold( 2 )

		iLegalCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_LEGAL')
		iSuzerainty = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_JUDGE')
		iLegalCivic = pPlayer.getCivics(iLegalCivicOption)

		if (iLegalCivic == iSuzerainty):
			pPID = pPlayer.getID()
			iGold = pPlayer.getGold( )
			if pPlayer.getGold( ) >= 5000:
				pPlayer.changeGold( 50 )
			if pPlayer.getGold( ) < 5000:
				if pPlayer.getGold( ) >= 100:
					pPlayer.changeGold( iGold//100 )
				else:
					pPlayer.changeGold( 1 )
					
## e-Bank and Suzerainty End ##

## Hereditary Vassal Bonus Start ##

		iTeam = pPlayer.getTeam()
		pTeam = gc.getTeam(iTeam)
		lPlayers = PyGame().getCivPlayerList( )

		iGovernmentCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_GOVERNMENT')
		iHereditary = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_HEREDITARY_RULE')
		iGovernmentCivic = pPlayer.getCivics(iGovernmentCivicOption)

		if (iGovernmentCivic == iHereditary):

			for iPlayer in range(gc.getMAX_PLAYERS()):
				ppPlayer = gc.getPlayer(iPlayer)
				if ( (ppPlayer.isAlive()==true) and (ppPlayer.isBarbarian()==false) ):
					if ( gc.getTeam(ppPlayer.getTeam()).isVassal(iTeam) == true ):

						iGold = ppPlayer.getGold( )
						if ppPlayer.getGold( ) >= 5000:
							ppPlayer.changeGold( 50 )
						if ppPlayer.getGold( ) < 5000:
							if ppPlayer.getGold( ) >= 100:
								ppPlayer.changeGold( iGold//50 )
							else:
								ppPlayer.changeGold( 2 )

						ppPlayer.changeCombatExperience( +1 )
	
## Hereditary Vassal Bonus End ## 

## End Times Start ##

		pTID = pPlayer.getTeam()

		iReligionCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_RELIGION')
		iEndTimes = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_APOC')
		iReligionCivic = pPlayer.getCivics(iReligionCivicOption)

		if (iReligionCivic == iEndTimes):

			basechance = 10
			estiEnd = CyGame().getEstimateEndTurn()
			if ( estiEnd >= 1000 ):
				basechance = 19
			elif ( estiEnd >= 700 ):
				basechance = 9
			elif ( estiEnd >= 5000 ):
				basechance = 7
			elif ( estiEnd >= 300 ):
				basechance = 5
			else:
				basechance = basechance


			chance = CyGame().getSorenRandNum(basechance, "free state religion spread chance")
			if ( chance == 0 ):
				lppCityUber5 = []
				for iiCity in range(pPlayer.getNumCities()):
					ppCity = pPlayer.getCity(iiCity)
					if ( not ppCity.isHasReligion(pPlayer.getStateReligion()) ):
						lppCityUber5.append(ppCity)
				if ( len(lppCityUber5) != 0 ):
					chance = CyGame().getSorenRandNum(len(lppCityUber5), "which city")
					ppCity = lppCityUber5[chance]
					ppCity.setHasReligion(pPlayer.getStateReligion(), true, true, true)

## End Times End ##

	def onEndPlayerTurn(self, argsList):
		'Called at the end of a players turn'
		iGameTurn, iPlayer = argsList
		
		if (gc.getGame().getElapsedGameTurns() == 1):
			if (gc.getPlayer(iPlayer).isHuman()):
				if (gc.getPlayer(iPlayer).canRevolution(0)):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_CHANGECIVIC)
					popupInfo.addPopup(iPlayer)
		
		CvAdvisorUtils.resetAdvisorNags()
		CvAdvisorUtils.endTurnFeats(iPlayer)

	def onEndTurnReady(self, argsList):
		iGameTurn = argsList[0]

	def onFirstContact(self, argsList):
		'Contact'
		iTeamX,iHasMetTeamY = argsList
		if (not self.__LOG_CONTACT):
			return
		CvUtil.pyPrint('Team %d has met Team %d' %(iTeamX, iHasMetTeamY))
	
	def onCombatResult(self, argsList):
		'Combat Result'
		pWinner,pLoser = argsList
		playerX = PyPlayer(pWinner.getOwner())
		unitX = PyInfo.UnitInfo(pWinner.getUnitType())
		playerY = PyPlayer(pLoser.getOwner())
		unitY = PyInfo.UnitInfo(pLoser.getUnitType())
		if (not self.__LOG_COMBAT):
			return
		if playerX and playerX and unitX and playerY:
			CvUtil.pyPrint('Player %d Civilization %s Unit %s has defeated Player %d Civilization %s Unit %s' 
				%(playerX.getID(), playerX.getCivilizationName(), unitX.getDescription(), 
				playerY.getID(), playerY.getCivilizationName(), unitY.getDescription()))

	def onCombatLogCalc(self, argsList):
		'Combat Result'	
		genericArgs = argsList[0][0]
		cdAttacker = genericArgs[0]
		cdDefender = genericArgs[1]
		iCombatOdds = genericArgs[2]
		CvUtil.combatMessageBuilder(cdAttacker, cdDefender, iCombatOdds)
		
	def onCombatLogHit(self, argsList):
		'Combat Message'
		global gCombatMessages, gCombatLog
		genericArgs = argsList[0][0]
		cdAttacker = genericArgs[0]
		cdDefender = genericArgs[1]
		iIsAttacker = genericArgs[2]
		iDamage = genericArgs[3]
		
		if cdDefender.eOwner == cdDefender.eVisualOwner:
			szDefenderName = gc.getPlayer(cdDefender.eOwner).getNameKey()
		else:
			szDefenderName = localText.getText("TXT_KEY_TRAIT_PLAYER_UNKNOWN", ())
		if cdAttacker.eOwner == cdAttacker.eVisualOwner:
			szAttackerName = gc.getPlayer(cdAttacker.eOwner).getNameKey()
		else:
			szAttackerName = localText.getText("TXT_KEY_TRAIT_PLAYER_UNKNOWN", ())

		if (iIsAttacker == 0):				
			combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_HIT", (szDefenderName, cdDefender.sUnitName, iDamage, cdDefender.iCurrHitPoints, cdDefender.iMaxHitPoints))
			CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
			CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
			if (cdDefender.iCurrHitPoints <= 0):
				combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_DEFEATED", (szAttackerName, cdAttacker.sUnitName, szDefenderName, cdDefender.sUnitName))
				CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
				CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
		elif (iIsAttacker == 1):
			combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_HIT", (szAttackerName, cdAttacker.sUnitName, iDamage, cdAttacker.iCurrHitPoints, cdAttacker.iMaxHitPoints))
			CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
			CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
			if (cdAttacker.iCurrHitPoints <= 0):
				combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_DEFEATED", (szDefenderName, cdDefender.sUnitName, szAttackerName, cdAttacker.sUnitName))
				CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
				CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)

	def onImprovementBuilt(self, argsList):
		'Improvement Built'
		iImprovement, iX, iY = argsList
		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Improvement %s was built at %d, %d'
			%(PyInfo.ImprovementInfo(iImprovement).getDescription(), iX, iY))

	def onImprovementDestroyed(self, argsList):
		'Improvement Destroyed'
		iImprovement, iOwner, iX, iY = argsList
		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Improvement %s was Destroyed at %d, %d'
			%(PyInfo.ImprovementInfo(iImprovement).getDescription(), iX, iY))

	def onRouteBuilt(self, argsList):
		'Route Built'
		iRoute, iX, iY = argsList
		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Route %s was built at %d, %d'
			%(gc.getRouteInfo(iRoute).getDescription(), iX, iY))

	def onPlotRevealed(self, argsList):
		'Plot Revealed'
		pPlot = argsList[0]
		iTeam = argsList[1]

	def onPlotFeatureRemoved(self, argsList):
		'Plot Revealed'
		pPlot = argsList[0]
		iFeatureType = argsList[1]
		pCity = argsList[2] # This can be null

	def onPlotPicked(self, argsList):
		'Plot Picked'
		pPlot = argsList[0]
		pPlotOwner = PyPlayer(pPlot.getOwner())
		CvUtil.pyPrint('Plot was picked at %d, %d'
			%(pPlot.getX(), pPlot.getY()))

## Great Bath / An Plague Check Start ##

		if pPlot.getOwner() != -1:
			if ( pPlotOwner.getStateReligion() == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ) or ( pPlotOwner.getStateReligion() != CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ):

#				lUnit = pPlotOwner.getUnitList()
#				for unit in lUnit:
#					if unit.getUnitType() == CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_STATESMAN"):
	
				pPlayer = gc.getPlayer(pPlot.getOwner())
				b_bath = gc.getInfoTypeForString("BUILDING_GREAT_BATH")
				obsoleteTech = gc.getBuildingInfo(b_bath).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_bath) == true:

							pAttackingUnit = GodsOfOld.iPushedButtonUnit

							iMessageID = self.m_iNetMessage_ProphetAn						
							iAttackingOwner = pAttackingUnit.getOwner( )
							iAttackingUnitID = pAttackingUnit.getID( )
							iDefendingOwner = pPlot.getOwner( )
							iDefendingUnitID = -1
			
							CyMessageControl( ).sendModNetMessage( iMessageID, iAttackingOwner, iAttackingUnitID, iDefendingOwner, iDefendingUnitID )
							return

## Great Bath / An Plague Check End ##

### Social Reform ###
		if GodsOfOld.iPushButton == 1:
			pUnit = GodsOfOld.iPushedButtonUnit
			
			iMessageID = self.m_iNetMessage_ProphetKi
			iPlotX = pPlot.getX()
			iPlotY = pPlot.getY()
			iOwner = pUnit.getOwner()
			iUnitID = pUnit.getID()
			
			pUnitOwner = gc.getPlayer(iOwner)
			if pUnitOwner.isTurnActive( ):

				CyMessageControl( ).sendModNetMessage( iMessageID, iPlotX, iPlotY, iOwner, iUnitID )
						
### Bubonic Plague ###

		if GodsOfOld.iPushButton == 3:
			pUnit = GodsOfOld.iPushedButtonUnit
						
			iMessageID = self.m_iNetMessage_ProphetEnlil
			iPlotX = pPlot.getX()
			iPlotY = pPlot.getY()
			iOwner = pUnit.getOwner()
			iUnitID = pUnit.getID()

			pUnitOwner = gc.getPlayer(iOwner)
			if pUnitOwner.isTurnActive( ):
					
				CyMessageControl( ).sendModNetMessage( iMessageID, iPlotX, iPlotY, iOwner, iUnitID )	

### Bubonic Plague ###
			
### Diplomacy ###

		if GodsOfOld.iPushButton == 4:
			pUnit = GodsOfOld.iPushedButtonUnit
						
			iMessageID = self.m_iNetMessage_ProphetNanna
			iPlotX = pPlot.getX()
			iPlotY = pPlot.getY()
			iOwner = pUnit.getOwner()
			iUnitID = pUnit.getID()

			pUnitOwner = gc.getPlayer(iOwner)
				
			if pUnitOwner.isTurnActive( ):
				CyMessageControl( ).sendModNetMessage( iMessageID, iPlotX, iPlotY, iOwner, iUnitID )	

### Diplomacy ###		
			
### Super Virus ###

		if GodsOfOld.iPushButton == 5:
			pUnit = GodsOfOld.iPushedButtonUnit
						
			iMessageID = self.m_iNetMessage_ProphetUtu
			iPlotX = pPlot.getX()
			iPlotY = pPlot.getY()
			iOwner = pUnit.getOwner()
			iUnitID = pUnit.getID()

			pUnitOwner = gc.getPlayer(iOwner)
			if pUnitOwner.isTurnActive( ):
				CyMessageControl( ).sendModNetMessage( iMessageID, iPlotX, iPlotY, iOwner, iUnitID )

### Super Virus ###				
			
	def onNukeExplosion(self, argsList):
		'Nuke Explosion'
		pPlot, pNukeUnit = argsList
		CvUtil.pyPrint('Nuke detonated at %d, %d'
			%(pPlot.getX(), pPlot.getY()))
		
		self.changeGameNumNukes(1)
		
	def onGotoPlotSet(self, argsList):
		'Nuke Explosion'
		pPlot, iPlayer = argsList

	def onBuildingBuilt(self, argsList):
		'Building Completed'
		pCity, iBuildingType = argsList
		game = CyGame()

### Via Start ###

		if ( iBuildingType == gc.getInfoTypeForString("BUILDING_VIA_APPIA") ):
			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			iPID = pPlayer.getID()
			iTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			b_via_appia = gc.getInfoTypeForString("BUILDING_VIA_APPIA")
			u_worker = gc.getInfoTypeForString( 'UNIT_WORKER' )
			p_morale = gc.getInfoTypeForString( 'PROMOTION_MORALE' )
			p_mobility = gc.getInfoTypeForString( 'PROMOTION_MOBILITY' )
			rt_roman_road = gc.getInfoTypeForString( 'ROUTE_ROMANROAD' )
			rt_railroad = gc.getInfoTypeForString( 'ROUTE_RAILROAD' )
			tt_noterrain = gc.getInfoTypeForString( 'NO_TERRAIN' )
			tt_coast = gc.getInfoTypeForString( 'TERRAIN_COAST' )
			tt_ocean = gc.getInfoTypeForString( 'TERRAIN_OCEAN' )

			for i in range(0):
				pNewUnit = pPlayer.initUnit( u_worker, iX, iY, UnitAITypes.UNITAI_WORKER, DirectionTypes.NO_DIRECTION )
			CyInterface().addMessage(iPID,false,15,CyTranslator().getText("TXT_KEY_VIA_APPIA_GAMETXT1",()),'AS2D_UNIT_BUILD_WORKER',0,gc.getUnitInfo(u_worker).getButton(),ColorTypes(44),iX,iY,True,True)
			radius = 2

			if ( gc.getNUM_CITY_PLOTS() > 21 ):
				radius = 3

			for iCity in range(pPlayer.getNumCities()):
				ppCity = pPlayer.getCity(iCity)
				iiX = ppCity.getX()
				iiY = ppCity.getY()
				for iXLoop in range(iiX - radius, iiX + radius + 1, 1):
					for iYLoop in range(iiY - radius, iiY + radius + 1, 1):
						pPlot = CyMap().plot(iXLoop, iYLoop)
						if ( pPlot.isPlayerCityRadius(iPID)==true ):
							if ( pPlot.getTeam()==iTID ):
								if ( pPlot.getRouteType()!=rt_roman_road and pPlot.getRouteType()!=rt_railroad ):
									if ( pPlot.getTerrainType()!=tt_coast and pPlot.getTerrainType()!=tt_ocean and pPlot.getTerrainType()!=tt_noterrain and pPlot.isPeak()==false ):
										pPlot.setRouteType(rt_roman_road)
										CyInterface().addMessage(iPID,True,-1,' ','',1,gc.getBuildingInfo(b_via_appia).getButton(),ColorTypes(5),iXLoop,iYLoop,True,True)


### Via End ###

## Great Bath Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_GREAT_BATH' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			pPlot = CyMap().plot(iX +0, iY +0)

			pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_RAINWATER_BASIN" ), 1)

## Great Bath End ##

### The School of Confucius Start ###

		if ( iBuildingType == gc.getInfoTypeForString("BUILDING_PALACE_CHINA") ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			iPID = pPlayer.getID()
			iTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			b_school = gc.getInfoTypeForString("BUILDING_PALACE_CHINA")
			u_prophet = gc.getInfoTypeForString( 'UNIT_PROPHET' )
			u_artist = gc.getInfoTypeForString( 'UNIT_ARTIST' )
			u_scientist = gc.getInfoTypeForString( 'UNIT_SCIENTIST' )
			u_merchant = gc.getInfoTypeForString( 'UNIT_MERCHANT' )
			u_engineer = gc.getInfoTypeForString( 'UNIT_ENGINEER' )
			u_general = gc.getInfoTypeForString( 'UNIT_GREAT_GENERAL' )
			u_spy = gc.getInfoTypeForString( 'UNIT_GREAT_SPY' )
			u_doctor = gc.getInfoTypeForString( 'UNIT_GREAT_DOCTOR' )
			u_statesman = gc.getInfoTypeForString( 'UNIT_STATESMAN' )
			u_citizen = gc.getInfoTypeForString( 'UNIT_CITIZEN' )

			self.iGreatPeopleNumberOne = self.getRandomNumber( 9 )

			for i in range(1):
				if self.iGreatPeopleNumberOne == 0:
					pNewUnit = pPlayer.initUnit( u_prophet, iX, iY, UnitAITypes.UNITAI_PROPHET, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_PROPHET", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 1:
					pNewUnit = pPlayer.initUnit( u_artist, iX, iY, UnitAITypes.UNITAI_ARTIST, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_ARTIST", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 2:
					pNewUnit = pPlayer.initUnit( u_scientist, iX, iY, UnitAITypes.UNITAI_SCIENTIST, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_SCIENTIST", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 3:
					pNewUnit = pPlayer.initUnit( u_merchant, iX, iY, UnitAITypes.UNITAI_MERCHANT, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_MERCHANT", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 4:
					pNewUnit = pPlayer.initUnit( u_engineer, iX, iY, UnitAITypes.UNITAI_ENGINEER, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_ENGINEER", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 5:
					pNewUnit = pPlayer.initUnit( u_general, iX, iY, UnitAITypes.UNITAI_GENERAL, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_GENERAL", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 6:
					pNewUnit = pPlayer.initUnit( u_spy, iX, iY, UnitAITypes.UNITAI_MERCHANT, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_SPY", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 7:
					pNewUnit = pPlayer.initUnit( u_doctor, iX, iY, UnitAITypes.UNITAI_GENERAL, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_DOCTOR", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 8:
					pNewUnit = pPlayer.initUnit( u_statesman, iX, iY, UnitAITypes.UNITAI_PROPHET, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_STATESMAN", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)
				if self.iGreatPeopleNumberOne == 9:
					pNewUnit = pPlayer.initUnit( u_citizen, iX, iY, UnitAITypes.UNITAI_SCIENTIST, DirectionTypes.NO_DIRECTION )
					szTitle = localText.getText( "TXT_KEY_SCHOOL_CITIZEN", ( ) )
					CyInterface().addImmediateMessage( szTitle , None)

### The School of Confucius End ###

### Machu Start ###

		if ( iBuildingType == gc.getInfoTypeForString("BUILDING_MACHU_PICCHU") ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			iPID = pPlayer.getID()
			iTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()

			### check inner city radius for Peaks (MachuPicchu Wonder prefers to be in the inner city radius of the home city of this wonder) ###
			lPeakPlotX = []
			lPeakPlotY = []
			for iXLoop in range(iX - 1, iX + 2, 1):
				for iYLoop in range(iY - 1, iY + 2, 1):
					pPlot = CyMap().plot(iXLoop, iYLoop)
					if ( pPlot.isPlayerCityRadius(iPID)==true ):
						if ( pPlot.getTeam()==iTID ):
							if ( pPlot.isPeak()==true  ):
								lPeakPlotX.append(iXLoop)
								lPeakPlotY.append(iYLoop)

			if ( len(lPeakPlotX) < 1 ):

				### check all city radius plots for Peaks if no inner peak available ###
				lPeakPlotX = []
				lPeakPlotY = []
				for iXLoop in range(iX - 2, iX + 3, 1):
					for iYLoop in range(iY - 2, iY + 3, 1):
						pPlot = CyMap().plot(iXLoop, iYLoop)
						if ( pPlot.isPlayerCityRadius(iPID)==true ):
							if ( pPlot.getTeam()==iTID ):
								if ( pPlot.isPeak()==true  ):
									lPeakPlotX.append(iXLoop)
									lPeakPlotY.append(iYLoop)

			if ( len(lPeakPlotX) > 0 ):
				it_machu_picchu = gc.getInfoTypeForString( "IMPROVEMENT_MACHU_PICCHU" )

				### set bonus for peaks in all city radius plots ###
				for iCity in range(pPlayer.getNumCities()):
					ppCity = pPlayer.getCity(iCity)
					iiX = ppCity.getX()
					iiY = ppCity.getY()
					### check all city radius plots for peak ###
					for iXLoop in range(iiX - 2, iiX + 3, 1):
						for iYLoop in range(iiY - 2, iiY + 3, 1):
							lPlot = CyMap().plot(iXLoop, iYLoop)
							if ( lPlot.isPlayerCityRadius(iPID)==true ):
								if ( lPlot.getTeam()==iTID ):
									if(lPlot.getTerrainType() != -1):
										if ( lPlot.isPeak()==true  ):
											lPlot.setImprovementType(it_machu_picchu)

				### now set bonus only with matchu picchu and remove matchu picchu (overlapping city radius) ###
				for iCity in range(pPlayer.getNumCities()):
					ppCity = pPlayer.getCity(iCity)
					iiX = ppCity.getX()
					iiY = ppCity.getY()
					### check all city radius plots for peak ###
					for iXLoop in range(iiX - 2, iiX + 3, 1):
						for iYLoop in range(iiY - 2, iiY + 3, 1):
							lPlot = CyMap().plot(iXLoop, iYLoop)
							if ( lPlot.isPlayerCityRadius(iPID)==true ):
								if ( lPlot.getTeam()==iTID ):
									if(lPlot.getTerrainType() != -1):
										if ( lPlot.isPeak()==true  ):
											if ( lPlot.getImprovementType() == it_machu_picchu ):
												CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_FOOD, 1)
												CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_PRODUCTION, 2)
												CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_COMMERCE, 1)
												lPlot.setImprovementType(-1)

												### ausgabe ###
												CyInterface().addMessage(iPID,True,0,' ','',1,'Art/Interface/Buttons/MachuPicchu_Python/MachuPicchu_Python_211.dds',ColorTypes(5), iXLoop, iYLoop,True,True)


				### set machu picchu on hill ###
				### get random Peak from list ###
				iRand = CyGame().getSorenRandNum( len(lPeakPlotX), "Random Peak")
				iXPeak = lPeakPlotX[iRand]
				iYPeak = lPeakPlotY[iRand]
				pPeakPlot = CyMap().plot(iXPeak, iYPeak)
				#pPeakPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FOREST" ), 1)
				pPeakPlot.setImprovementType(gc.getInfoTypeForString( "IMPROVEMENT_MACHU_PICCHU" ))
				CyGame().setPlotExtraYield(iXPeak, iYPeak, YieldTypes.YIELD_FOOD, 1)
				CyGame().setPlotExtraYield(iXPeak, iYPeak, YieldTypes.YIELD_COMMERCE, 6)
				plot_culture_1_10 = int(pPeakPlot.getCulture(iPID) * 0.10)
				pPeakPlot.changeCulture(iPID, plot_culture_1_10, true)

				### ausgabe ###
				CyInterface().addMessage(iPID,false,10,CyTranslator().getText("TXT_KEY_MACHU_PICCHU_GAMETXT1",(iXPeak,iXPeak)),'',0,gc.getBuildingInfo(gc.getInfoTypeForString("BUILDING_MACHU_PICCHU")).getButton(),ColorTypes(5), iXPeak, iYPeak,True,True)
				### message: The Machu Picchu provides some benefits from Peaks! ###

				### set additional bonus for peaks in the wonder city radius ###
				for iXLoop in range(iX - 2, iX + 3, 1):
					for iYLoop in range(iY - 2, iY + 3, 1):
						pPlot = CyMap().plot(iXLoop, iYLoop)
						if ( pPlot.isPlayerCityRadius(iPID)==true ):
							if ( pPlot.getTeam()==iTID ):
								if(pPlot.getTerrainType() != -1):
									if ( pPlot.isPeak()==true  ):
											CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_COMMERCE, 0)

											### ausgabe ###
											if ( iXLoop != iXPeak or iYLoop != iYPeak ):
												CyInterface().addMessage(iPID,True,0," ",'0',1,'Art/Interface/Buttons/MachuPicchu_Python/MachuPicchu_Python_212.dds',ColorTypes(5), iXLoop, iYLoop,True,True)
											else:
												CyInterface().addMessage(iPID,True,0," ",'0',1,'Art/Interface/Buttons/MachuPicchu_Python/MachuPicchu_Python_218.dds',ColorTypes(5), iXLoop, iYLoop,True,True)
				
### Machu End ###

## Fusion Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_FUSIONSREAKTOR' ):
			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			wonderteam = gc.getTeam(pCity.getOwner())
			t_fusion = gc.getInfoTypeForString("TECH_FUSION")
			iX = pCity.getX()
			iY = pCity.getY()
			CityName = pCity.getName()
			nachricht = 0
			research_cost = wonderteam.getResearchCost(t_fusion)
			research_progress = wonderteam.getResearchProgress(t_fusion)
			free_research_10 = int(research_cost / 10)
			free_research_90 = free_research_10 * 5
			if ( research_progress < free_research_10 ):
				wonderteam.changeResearchProgress(t_fusion, free_research_90, pPID)
				nachricht = 1
			elif ( research_progress >= free_research_10 and wonderteam.isHasTech(t_fusion) == false ):
				wonderteam.changeResearchProgress(t_fusion, free_research_90, pPID)
				uberfullung = ( research_progress + free_research_90 ) - research_cost
				if ( uberfullung <= 0 ):
					uberfullung = 100
				free_research = int(free_research_90 - uberfullung)
				uberfullung = int(uberfullung / 10)
				pPlayer.changeGold(uberfullung)
				nachricht = 2
			elif ( wonderteam.isHasTech(t_fusion) == true ):
				uberfullung = int(research_cost / 10)
				pPlayer.changeGold(uberfullung)
				nachricht = 3		
			if ( nachricht == 1 ):
				CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_FUSIONSREAKTOR_GAMETXT1",(CityName,free_research_90)),'',0,'Art/Fusionsreaktor/NuclearPower.dds',ColorTypes(44), iX, iY, True,True)
				### message: The Fusion Plant in %s1 provides 90% (%d2) of the Research Costs for the Fusion Technology! ###
			elif ( nachricht == 2 ):
				CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_FUSIONSREAKTOR_GAMETXT2",(CityName,free_research,uberfullung)),'',0,'Art/Fusionsreaktor/NuclearPower.dds',ColorTypes(44), iX, iY, True,True)
				### message: The Fusion Plant in %s1 provides %d2 Tech Points for the Fusion Technology and %d3 Gold! ###
			elif ( nachricht == 3 ):
				CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_FUSIONSREAKTOR_GAMETXT3",(CityName,uberfullung)),'',0,'Art/Fusionsreaktor/NuclearPower.dds',ColorTypes(44), iX, iY, True,True)
				### message: The Fusion Plant in %s1 provides %d2 Gold! ###

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_FUSIONSREAKTOR' ):
			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			wonderteam = gc.getTeam(pCity.getOwner())
			t_fusion = gc.getInfoTypeForString("TECH_COLD_FUSION")
			iX = pCity.getX()
			iY = pCity.getY()
			CityName = pCity.getName()
			nachricht = 0
			research_cost = wonderteam.getResearchCost(t_fusion)
			research_progress = wonderteam.getResearchProgress(t_fusion)
			free_research_10 = int(research_cost / 10)
			free_research_90 = free_research_10 * 5
			if ( research_progress < free_research_10 ):
				wonderteam.changeResearchProgress(t_fusion, free_research_90, pPID)
				nachricht = 1
			elif ( research_progress >= free_research_10 and wonderteam.isHasTech(t_fusion) == false ):
				wonderteam.changeResearchProgress(t_fusion, free_research_90, pPID)
				uberfullung = ( research_progress + free_research_90 ) - research_cost
				if ( uberfullung <= 0 ):
					uberfullung = 100
				free_research = int(free_research_90 - uberfullung)
				uberfullung = int(uberfullung / 10)
				pPlayer.changeGold(uberfullung)
				nachricht = 2
			elif ( wonderteam.isHasTech(t_fusion) == true ):
				uberfullung = int(research_cost / 10)
				pPlayer.changeGold(uberfullung)
				nachricht = 3		
			if ( nachricht == 1 ):
				CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_FUSIONSREAKTOR_GAMETXT11",(CityName,free_research_90)),'',0,'Art/Fusionsreaktor/NuclearPower.dds',ColorTypes(44), iX, iY, True,True)
				### message: The Fusion Plant in %s1 provides 90% (%d2) of the Research Costs for the Fusion Technology! ###
			elif ( nachricht == 2 ):
				CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_FUSIONSREAKTOR_GAMETXT22",(CityName,free_research,uberfullung)),'',0,'Art/Fusionsreaktor/NuclearPower.dds',ColorTypes(44), iX, iY, True,True)
				### message: The Fusion Plant in %s1 provides %d2 Tech Points for the Fusion Technology and %d3 Gold! ###
			elif ( nachricht == 3 ):
				CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_FUSIONSREAKTOR_GAMETXT33",(CityName,uberfullung)),'',0,'Art/Fusionsreaktor/NuclearPower.dds',ColorTypes(44), iX, iY, True,True)
				### message: The Fusion Plant in %s1 provides %d2 Gold! ###

## Fusion End ##

## Tesla Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_TESLA' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			plotX = pCity.getX()
			plotY = pCity.getY()

			earthquakeEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "EFFECT_PING" )
			#CyEngine( ).triggerEffect( earthquakeEffect, plot.getPoint( ) )
			CyEngine( ).triggerEffect( earthquakeEffect, gc.getMap( ).plot( pCity.getX() + 0, pCity.getY() + 0 ).getPoint( ) )


## Tesla End ##

## Djenne Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_DJENNE' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			tt_desert = gc.getInfoTypeForString( 'TERRAIN_DESERT' )

			for iXLoop in range(iX - 2, iX + 3, 1):
				for iYLoop in range(iY - 2, iY + 3, 1):
					pPlot = CyMap().plot(iXLoop, iYLoop)
					if ( pPlot.isPlayerCityRadius(pPID)==true ):
						if ( pPlot.getTeam()==pTID ):
							if ( pPlot.getTerrainType()==tt_desert ):
								CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_COMMERCE, 2)
			
			
			CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_DJENNE_PYTHON",()),'',0,'Art/Djenne.dds',ColorTypes(44), iX, iY, True,True)


## Djenne End ##

## Ice Hotel Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_ICE_HOTEL' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			tt_desert = gc.getInfoTypeForString( 'TERRAIN_SNOW' )
			tt_tundra = gc.getInfoTypeForString( 'TERRAIN_TUNDRA' )
			tt_ocean = gc.getInfoTypeForString( 'TERRAIN_OCEAN' )
			tt_coast = gc.getInfoTypeForString( 'TERRAIN_COAST' )

			for iXLoop in range(iX - 2, iX + 3, 1):
				for iYLoop in range(iY - 2, iY + 3, 1):
					pPlot = CyMap().plot(iXLoop, iYLoop)
					if ( pPlot.isPlayerCityRadius(pPID)==true ):
						if ( pPlot.getTeam()==pTID ):
							if ( pPlot.getTerrainType()==tt_desert ):
								CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_COMMERCE, 2)
								if( (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_MARBLE' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_STONE' ))):
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_QUARRY' ))
								elif( pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_OIL' ) ):
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WELL' ))
								elif( (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_ALUMINUM' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_COAL' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_COPPER' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_IRON' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_URANIUM' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_GEMS' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_GOLD' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_SILVER' ))):
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_MINE' ))
								elif( (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_DEER' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_FUR' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_IVORY' )) ):
									CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_FOOD, 1)
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_CAMP' ))
									pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FOREST" ), 1)
								elif ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_PERMA' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WINDMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_MINE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WATERMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_IGLOO' )):
									pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FOREST" ), 1)
							if ( pPlot.getTerrainType()==tt_tundra ):
								CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_COMMERCE, 2)
								if( (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_MARBLE' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_STONE' ))):
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_QUARRY' ))
								elif( pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_OIL' ) ):
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WELL' ))
								elif( (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_ALUMINUM' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_COAL' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_COPPER' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_IRON' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_URANIUM' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_GEMS' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_GOLD' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_SILVER' ))):
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_MINE' ))
								elif( (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_DEER' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_FUR' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_IVORY' )) ):
									CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_FOOD, 1)
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_CAMP' ))
									pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FOREST" ), 1)
								elif ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_PERMA' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WINDMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_MINE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WATERMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_IGLOO' )):
									pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FOREST" ), 1)
							if ( pPlot.getTerrainType()==tt_ocean ) or ( pPlot.getTerrainType()==tt_coast ):
								if ( pPlot.getFeatureType()==gc.getInfoTypeForString( 'FEATURE_ICE' )):
									pPlot.setFeatureType(-1, 0)
								elif( (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_FISH' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_CLAM' )) or (pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_CRAB' )) ):
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_FISHING_BOATS' ))
									CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_FOOD, 1)
								elif( pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_OIL' ) ) and ( gc.getTeam(pTID).isHasTech(gc.getInfoTypeForString("TECH_PLASTICS")) == true ):
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_OFFSHORE_PLATFORM' ))
								elif( pPlot.getBonusType(-1)==gc.getInfoTypeForString( 'BONUS_WHALE' ) ):
									pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WHALING_BOATS' ))
									CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_FOOD, 1)
			
			
			CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_ICE_HOTEL_PYTHON",()),'',0,'Art/Ice Hotel.dds',ColorTypes(44), iX, iY, True,True)

## Ice Hotel End ##

## Public Works Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_PUBLIC_WORKS' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			pPlot = CyMap().plot(iX +0, iY +0)
			iChange = +1

			BBuilding	= "BUILDING_BARRACKS"
			B2Building	= "BUILDING_TRIBAL"
			B3Building	= "BUILDING_ZULU_IKHANDA"
			B4Building	= "BUILDING_ABORIGINES_BORARING"

			GBuilding	= "BUILDING_GRANARY"
			G2Building	= "BUILDING_INCAN_TERRACE"
			G3Building	= "BUILDING_NUBIAN"
			G4Building	= "BUILDING_TUPI"

			LBuilding	= "BUILDING_LIBRARY"
			L2Building	= "BUILDING_ARABIAN_MADRASSA"
			L4Building	= "BUILDING_ASS"

			CBuilding	= "BUILDING_COURTHOUSE"
			C2Building	= "BUILDING_COURTHOUSE_ZAM"
			C3Building	= "BUILDING_AZTEC_SACRIFICIAL_ALTAR"
			C4Building	= "BUILDING_HOLY_ROMAN_RATHAUS"
			C5Building	= "BUILDING_SUMERIAN_ZIGGURAT"
			C6Building	= "BUILDING_KI"

			MBuilding	= "BUILDING_MARKET"
			M2Building	= "BUILDING_ROMAN_FORUM"
			M3Building	= "BUILDING_TREE"
			M4Building	= "BUILDING_AKKADIAN"

			FBuilding	= "BUILDING_FORGE"
			F2Building	= "BUILDING_MALI_MINT"
			F3Building	= "BUILDING_IFORGE"
			F4Building	= "BUILDING_KILN"

			ABuilding	= "BUILDING_AQUEDUCT"
			A2Building	= "BUILDING_OTTOMAN_HAMMAM"
			A3Building	= "BUILDING_KHMER_BARAY"
			A4Building	= "BUILDING_BERBER"

			DBuilding	= "BUILDING_THEATRE"
			D2Building	= "BUILDING_CHINESE_PAVILLION"
			D3Building	= "BUILDING_BYZANTINE_HIPPODROME"

			HBuilding	= "BUILDING_HARBOR"
			H2Building	= "BUILDING_FISHY"
			H3Building	= "BUILDING_CARTHAGE_COTHON"

			LHBuilding	= "BUILDING_LIGHTHOUSE"
			LH2Building	= "BUILDING_VIKING_TRADING_POST"

			if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_CHINA")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(D2Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_BYZANTIUM")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(D3Building), 1)
			elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_CHINA")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_BYZANTIUM")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(DBuilding), 1)

			if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_BARBARIAN")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(B2Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_ZULU")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(B3Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_ABORIGINES")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(B4Building), 1)
			elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_BARBARIAN")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_ZULU")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_ABORIGINES")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(BBuilding), 1)

			if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_INCA")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(G2Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_NUBIA")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(G3Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_TUPI")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(G4Building), 1)
			elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_INCA")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_NUBIA")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_TUPI")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(GBuilding), 1)

			if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_ARABIA")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(L2Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_ASSYRIA")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(L4Building), 1)
			elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_ARABIA")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_ASSYRIA")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(LBuilding), 1)

			if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_ROME")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(M2Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_IRELAND")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(M3Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_AKKAD")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(M4Building), 1)
			elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_ROME")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_IRELAND")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_AKKAD")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(MBuilding), 1)

			if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_MALI")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(F2Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_HITTITES")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(F3Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_HURR")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(F4Building), 1)
			elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_MALI")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_HITTITES")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_HURR")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(FBuilding), 1)

			if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_OTTOMAN")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(A2Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_KHMER")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(A3Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_BERBER")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(A4Building), 1)
				pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FOSSIL_WATER" ), 1)
			elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_OTTOMAN")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_KHMER")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_BERBER")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(ABuilding), 1)

			if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_AZTEC")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(C3Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_TIM")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(C2Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_HOLY_ROMAN")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(C4Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_SUMERIA")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(C5Building), 1)
			elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_MINO")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(C6Building), 1)
			elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_AZTEC")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_TIM")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_HOLY_ROMAN")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_SUMERIA")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_MINO")):
				pCity.setNumRealBuilding(gc.getInfoTypeForString(CBuilding), 1)

#			if pCity.isCoastal():
#
#				if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_POLYNESIA")):
#					pCity.setNumRealBuilding(gc.getInfoTypeForString(H2Building), 1)
#				elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_CARTHAGE")):
#					pCity.setNumRealBuilding(gc.getInfoTypeForString(H3Building), 1)
#				elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_POLYNESIA")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_CARTHAGE")):
#					pCity.setNumRealBuilding(gc.getInfoTypeForString(HBuilding), 1)
#
#				if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_VIKING")):
#					pCity.setNumRealBuilding(gc.getInfoTypeForString(L2HBuilding), 1)
#				elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_VIKING")):
#					pCity.setNumRealBuilding(gc.getInfoTypeForString(LHBuilding), 1)


			pCity.changePopulation(iChange)

			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()

			for iXLoop in range(iX - 2, iX + 3, 1):
				for iYLoop in range(iY - 2, iY + 3, 1):
					pPlot = CyMap().plot(iXLoop, iYLoop)
					if ( pPlot.isPlayerCityRadius(pPID)==true ):
						if ( pPlot.getTeam()==pTID ):
							if ( pPlot.getImprovementType()==gc.getInfoTypeForString( 'IMPROVEMENT_VILLAGE' )):
								pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_TOWN' ))
							elif ( pPlot.getImprovementType()==gc.getInfoTypeForString( 'IMPROVEMENT_HAMLET' )):
								pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_VILLAGE' ))
							elif ( pPlot.getImprovementType()==gc.getInfoTypeForString( 'IMPROVEMENT_COTTAGE' )):
								pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_HAMLET' ))
							elif ( pPlot.getImprovementType()==gc.getInfoTypeForString( 'IMPROVEMENT_FORT' )):
								pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_FORT2' ))
							elif ( pPlot.getImprovementType()==gc.getInfoTypeForString( 'IMPROVEMENT_SUBMERGED_TOWN' )):
								pPlot.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_SUBMERGED_TOWN2' ))
						
			CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_PUBLIC_PYTHON",()),'',0,'Art/Public Works.dds',ColorTypes(44), iX, iY, True,True)

## Public Works End ##

## Sweden Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_SWEDEN' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			iGold = pPlayer.getGold( )
			
			if pPlayer.getGold( ) < 10000:
				pPlayer.changeGold( iGold//10 )
			else:
				pPlayer.changeGold( 1000)

## Sweden End ##

## E Bank Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_E_BANK' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			iGold = pPlayer.getGold( )
			pPlayer.changeGold( iGold//2 )

## E Bank End ##

## Bridge Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_BRIDGE' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			tt_ocean = gc.getInfoTypeForString( 'TERRAIN_OCEAN' )
			tt_coast = gc.getInfoTypeForString( 'TERRAIN_COAST' )

			pPlotX1 = CyMap().plot(iX +1, iY +0)
			pPlotX2 = CyMap().plot(iX +2, iY +0)
			pPlotX3 = CyMap().plot(iX +3, iY +0)
			pPlotX4 = CyMap().plot(iX +4, iY +0)
			pPlotX5 = CyMap().plot(iX +5, iY +0)

			pPlotNX1 = CyMap().plot(iX -1, iY +0)
			pPlotNX2 = CyMap().plot(iX -2, iY +0)
			pPlotNX3 = CyMap().plot(iX -3, iY +0)
			pPlotNX4 = CyMap().plot(iX -4, iY +0)
			pPlotNX5 = CyMap().plot(iX -5, iY +0)

			pPlotY1 = CyMap().plot(iX +0, iY +1)
			pPlotY2 = CyMap().plot(iX +0, iY +2)
			pPlotY3 = CyMap().plot(iX +0, iY +3)
			pPlotY4 = CyMap().plot(iX +0, iY +4)
			pPlotY5 = CyMap().plot(iX +0, iY +5)

			pPlotNY1 = CyMap().plot(iX +0, iY -1)
			pPlotNY2 = CyMap().plot(iX +0, iY -2)
			pPlotNY3 = CyMap().plot(iX +0, iY -3)
			pPlotNY4 = CyMap().plot(iX +0, iY -4)
			pPlotNY5 = CyMap().plot(iX +0, iY -5)

			if ( pPlotX1.getTeam()==pTID ) or ( pPlotX1.getTeam()!=pTID ):
				if ( pPlotX1.getTerrainType()==tt_coast ) or ( pPlotX1.getTerrainType()==tt_ocean ):
					if ( pPlotX2.getTeam()==pTID ) or ( pPlotX2.getTeam()!=pTID ):
						if ( pPlotX2.getTerrainType()!=tt_coast ) and ( pPlotX2.getTerrainType()!=tt_ocean ):
							#pPlotX1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
							pPlotX1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
						elif ( pPlotX2.getTerrainType()==tt_coast ) or ( pPlotX2.getTerrainType()==tt_ocean ):
								if ( pPlotX3.getTeam()==pTID ) or ( pPlotX3.getTeam()!=pTID ):
									if ( pPlotX3.getTerrainType()!=tt_coast ) and ( pPlotX3.getTerrainType()!=tt_ocean ):
										#pPlotX1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
										pPlotX1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
										#pPlotX2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
										pPlotX2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
									elif ( pPlotX3.getTerrainType()==tt_coast ) or ( pPlotX3.getTerrainType()==tt_ocean ):
										if ( pPlotX4.getTeam()==pTID ) or ( pPlotX4.getTeam()!=pTID ):
											if ( pPlotX4.getTerrainType()!=tt_coast ) and ( pPlotX4.getTerrainType()!=tt_ocean ):
												#pPlotX1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotX1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
												#pPlotX2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotX2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
												#pPlotX3.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotX3.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
											elif ( pPlotX4.getTerrainType()==tt_coast ) or ( pPlotX4.getTerrainType()==tt_ocean ):
												if ( pPlotX5.getTeam()==pTID ) or ( pPlotX5.getTeam()!=pTID ):
													if ( pPlotX5.getTerrainType()!=tt_coast ) and ( pPlotX5.getTerrainType()!=tt_ocean ):
														#pPlotX1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotX1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
														#pPlotX2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotX2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
														#pPlotX3.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotX3.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
														#pPlotX4.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotX4.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
													elif ( pPlotX5.getTerrainType()==tt_coast ) or ( pPlotX5.getTerrainType()==tt_ocean ):
														CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_BRIDGE_FAIL_PYTHON",()),'',0,'Art/Bridge Button.dds',ColorTypes(44), iX, iY, True,True)

			if ( pPlotNX1.getTeam()==pTID ) or ( pPlotNX1.getTeam()!=pTID ):
				if ( pPlotNX1.getTerrainType()==tt_coast ) or ( pPlotNX1.getTerrainType()==tt_ocean ):
					if ( pPlotNX2.getTeam()==pTID ) or ( pPlotNX2.getTeam()!=pTID ):
						if ( pPlotNX2.getTerrainType()!=tt_coast ) and ( pPlotNX2.getTerrainType()!=tt_ocean ):
							#pPlotNX1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
							pPlotNX1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
						elif ( pPlotNX2.getTerrainType()==tt_coast ) or ( pPlotNX2.getTerrainType()==tt_ocean ):
								if ( pPlotNX3.getTeam()==pTID ) or ( pPlotNX3.getTeam()!=pTID ):
									if ( pPlotNX3.getTerrainType()!=tt_coast ) and ( pPlotNX3.getTerrainType()!=tt_ocean ):
										#pPlotNX1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
										pPlotNX1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
										#pPlotNX2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
										pPlotNX2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
									elif ( pPlotNX3.getTerrainType()==tt_coast ) or ( pPlotNX3.getTerrainType()==tt_ocean ):
										if ( pPlotNX4.getTeam()==pTID ) or ( pPlotNX4.getTeam()!=pTID ):
											if ( pPlotNX4.getTerrainType()!=tt_coast ) and ( pPlotNX4.getTerrainType()!=tt_ocean ):
												#pPlotNX1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotNX1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
												#pPlotNX2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotNX2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
												#pPlotNX3.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotNX3.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
											elif ( pPlotNX4.getTerrainType()==tt_coast ) or ( pPlotNX4.getTerrainType()==tt_ocean ):
												if ( pPlotNX5.getTeam()==pTID ) or ( pPlotNX5.getTeam()!=pTID ):
													if ( pPlotNX5.getTerrainType()!=tt_coast ) and ( pPlotNX5.getTerrainType()!=tt_ocean ):
														#pPlotNX1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotNX1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
														#pPlotNX2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotNX2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
														#pPlotNX3.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotNX3.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
														#pPlotNX4.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotNX4.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_WATER_BRIDGE_WEST' ))
													elif ( pPlotNX5.getTerrainType()==tt_coast ) or ( pPlotNX5.getTerrainType()==tt_ocean ):
														CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_BRIDGE_FAIL_PYTHON",()),'',0,'Art/Bridge Button.dds',ColorTypes(44), iX, iY, True,True)

			if ( pPlotY1.getTeam()==pTID ) or ( pPlotY1.getTeam()!=pTID ):
				if ( pPlotY1.getTerrainType()==tt_coast ) or ( pPlotY1.getTerrainType()==tt_ocean ):
					if ( pPlotY2.getTeam()==pTID ) or ( pPlotY2.getTeam()!=pTID ):
						if ( pPlotY2.getTerrainType()!=tt_coast ) and ( pPlotY2.getTerrainType()!=tt_ocean ):
							#pPlotY1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
							pPlotY1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
						elif ( pPlotY2.getTerrainType()==tt_coast ) or ( pPlotY2.getTerrainType()==tt_ocean ):
								if ( pPlotY3.getTeam()==pTID ) or ( pPlotY3.getTeam()!=pTID ):
									if ( pPlotY3.getTerrainType()!=tt_coast ) and ( pPlotY3.getTerrainType()!=tt_ocean ):
										#pPlotY1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
										pPlotY1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
										#pPlotY2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
										pPlotY2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
									elif ( pPlotY3.getTerrainType()==tt_coast ) or ( pPlotY3.getTerrainType()==tt_ocean ):
										if ( pPlotY4.getTeam()==pTID ) or ( pPlotY4.getTeam()!=pTID ):
											if ( pPlotY4.getTerrainType()!=tt_coast ) and ( pPlotY4.getTerrainType()!=tt_ocean ):
												#pPlotY1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotY1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
												#pPlotY2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotY2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
												#pPlotY3.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotY3.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
											elif ( pPlotY4.getTerrainType()==tt_coast ) or ( pPlotY4.getTerrainType()==tt_ocean ):
												if ( pPlotY5.getTeam()==pTID ) or ( pPlotY5.getTeam()!=pTID ):
													if ( pPlotY5.getTerrainType()!=tt_coast ) and ( pPlotY5.getTerrainType()!=tt_ocean ):
														#pPlotY1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotY1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
														#pPlotY2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotY2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
														#pPlotY3.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotY3.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
														#pPlotY4.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotY4.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
													elif ( pPlotY5.getTerrainType()==tt_coast ) or ( pPlotY5.getTerrainType()==tt_ocean ):
														CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_BRIDGE_FAIL_PYTHON",()),'',0,'Art/Bridge Button.dds',ColorTypes(44), iX, iY, True,True)

			if ( pPlotNY1.getTeam()==pTID ) or ( pPlotNY1.getTeam()!=pTID ):
				if ( pPlotNY1.getTerrainType()==tt_coast ) or ( pPlotNY1.getTerrainType()==tt_ocean ):
					if ( pPlotNY2.getTeam()==pTID ) or ( pPlotNY2.getTeam()!=pTID ):
						if ( pPlotNY2.getTerrainType()!=tt_coast ) and ( pPlotNY2.getTerrainType()!=tt_ocean ):
							#pPlotNY1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
							pPlotNY1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
						elif ( pPlotNY2.getTerrainType()==tt_coast ) or ( pPlotNY2.getTerrainType()==tt_ocean ):
								if ( pPlotNY3.getTeam()==pTID ) or ( pPlotNY3.getTeam()!=pTID ):
									if ( pPlotNY3.getTerrainType()!=tt_coast ) and ( pPlotNY3.getTerrainType()!=tt_ocean ):
										#pPlotNY1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
										pPlotNY1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
										#pPlotNY2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
										pPlotNY2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
									elif ( pPlotNY3.getTerrainType()==tt_coast ) or ( pPlotNY3.getTerrainType()==tt_ocean ):
										if ( pPlotNY4.getTeam()==pTID ) or ( pPlotNY4.getTeam()!=pTID ):
											if ( pPlotNY4.getTerrainType()!=tt_coast ) and ( pPlotNY4.getTerrainType()!=tt_ocean ):
												#pPlotNY1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotNY1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
												#pPlotNY2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotNY2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
												#pPlotNY3.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
												pPlotNY3.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
											elif ( pPlotNY4.getTerrainType()==tt_coast ) or ( pPlotNY4.getTerrainType()==tt_ocean ):
												if ( pPlotNY5.getTeam()==pTID ) or ( pPlotNY5.getTeam()!=pTID ):
													if ( pPlotNY5.getTerrainType()!=tt_coast ) and ( pPlotNY5.getTerrainType()!=tt_ocean ):
														#pPlotNY1.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotNY1.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
														#pPlotNY2.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotNY2.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
														#pPlotNY3.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotNY3.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
														#pPlotNY4.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS2" ), 1, 1)
														pPlotNY4.setImprovementType(gc.getInfoTypeForString( 'IMPROVEMENT_BRIDGE' ))
													elif ( pPlotNY5.getTerrainType()==tt_coast ) or ( pPlotNY5.getTerrainType()==tt_ocean ):
														CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_BRIDGE_FAIL_PYTHON",()),'',0,'Art/Bridge Button.dds',ColorTypes(44), iX, iY, True,True)
			
			CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_BRIDGE_PYTHON",()),'',0,'Art/Bridge Button.dds',ColorTypes(44), iX, iY, True,True)

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_BRIDGE' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pCityName = pCity.getName()
			iX = pCity.getX()
			iY = pCity.getY()

			pCC = pCity.getCulture(pPID)
			pCC = pCC / 5
			pCC = int(pCC + 0.5)

			pCity.changeCulture(pPID, pCC, true)

			CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_GOLDEN_GATE_CULTURE2",(pCC, pCityName)),'',0,'Art/Bridge Button.dds',ColorTypes(24), -1, -1,True,True)

## Bridge End ##

## Berber Start ##

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_BERBER' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			pPlot = CyMap().plot(iX +0, iY +0)

			pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FOSSIL_WATER" ), 1)

## Berber End ##

		if ((pCity.getOwner() == gc.getGame().getActivePlayer())):
			# If this is a wonder...
			popupInfo = CyPopupInfo()
			popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
			popupInfo.setData1(iBuildingType)
			popupInfo.setData2(pCity.getID())
			popupInfo.setData3(0)
			popupInfo.setText(u"showWonderMovie")
			popupInfo.addPopup(pCity.getOwner())

		CvAdvisorUtils.buildingBuiltFeats(pCity, iBuildingType)

### Force Shielding ###
		
		if iBuildingType == CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ARCOLOGY_SHIELDING"):
			pCity.setNumRealBuilding(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ARCOLOGY"), False)
		elif iBuildingType == CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ADVANCED_SHIELDING"):
			pCity.setNumRealBuilding(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ARCOLOGY_SHIELDING"), False)
		
		if iBuildingType == CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ADVANCED_SHIELDING3"):
			pCity.setNumRealBuilding(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ARCOLOGY"), False)
		
		if iBuildingType == CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ADVANCED_SHIELDING3"):
			pCity.setNumRealBuilding(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ARCOLOGY_SHIELDING"), False)
		
		if iBuildingType == CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ADVANCED_SHIELDING3"):
			pCity.setNumRealBuilding(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_ADVANCED_SHIELDING"), False)

### Force Shielding ###

		if (not self.__LOG_BUILDING):
			return
		CvUtil.pyPrint('%s was finished by Player %d Civilization %s' 
			%(PyInfo.BuildingInfo(iBuildingType).getDescription(), pCity.getOwner(), gc.getPlayer(pCity.getOwner()).getCivilizationDescription(0)))
	
	def onProjectBuilt(self, argsList):
		'Project Completed'
		pCity, iProjectType = argsList
		game = CyGame()

## Eden Start ##

		if iProjectType == gc.getInfoTypeForString('PROJECT_TERRA'):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			tt_desert = gc.getInfoTypeForString( 'TERRAIN_DESERT' )
			tt_plain = gc.getInfoTypeForString( 'TERRAIN_PLAINS' )
			tt_grass = gc.getInfoTypeForString( 'TERRAIN_GRASS' )
			tt_tundra = gc.getInfoTypeForString( 'TERRAIN_TUNDRA' )
			tt_snow = gc.getInfoTypeForString( 'TERRAIN_SNOW' )
			tt_ocean = gc.getInfoTypeForString( 'TERRAIN_OCEAN' )
			tt_marsh = gc.getInfoTypeForString( 'TERRAIN_MARSH' )
			

			for iXLoop in range(iX - 50, iX + 50, 1):
				for iYLoop in range(iY - 50, iY + 50, 1):
					pPlot = CyMap().plot(iXLoop, iYLoop)
					#if ( pPlot.isPlayerCityRadius(pPID)==true ):
					if ( pPlot.getTeam()==pTID ):
						if ( pPlot.getTerrainType()==tt_grass ):
							if ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_FARM' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WINDMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_MINE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WATERMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WORKSHOP' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_LUMBERMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_PLANTATION' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_QUARRY' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_PASTURE' ))and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_CAMP' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WELL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WINERY' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_COTTAGE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_HAMLET' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_VILLAGE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_TOWN' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_FOREST_PRESERVE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_FORT' )):
								if ( pPlot.getFeatureType()!=gc.getInfoTypeForString( 'FEATURE_JUNGLE' )):	
									pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FOREST" ), 1)
						elif ( pPlot.getTerrainType()==tt_plain ):
							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_marsh ):
							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_GRASS" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_tundra ):
							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_PLAINS" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_snow ):
							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_TUNDRA" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_ocean ):
							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_desert ):
							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_PLAINS" ), 1, 1)
			
			CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_EDEN_PYTHON",()),'',0,'Art/naturopath.dds',ColorTypes(44), iX, iY, True,True)

## Eden End ##

## Apocalypse Start ##

		if iProjectType == gc.getInfoTypeForString('PROJECT_APOC'):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			tt_desert = gc.getInfoTypeForString( 'TERRAIN_DESERT' )
			tt_plain = gc.getInfoTypeForString( 'TERRAIN_PLAINS' )
			tt_grass = gc.getInfoTypeForString( 'TERRAIN_GRASS' )
			tt_tundra = gc.getInfoTypeForString( 'TERRAIN_TUNDRA' )
			tt_snow = gc.getInfoTypeForString( 'TERRAIN_SNOW' )
			tt_ocean = gc.getInfoTypeForString( 'TERRAIN_OCEAN' )
			tt_coast = gc.getInfoTypeForString( 'TERRAIN_COAST' )
			tt_marsh = gc.getInfoTypeForString( 'TERRAIN_MARSH' )

			for iXLoop in range(iX - 45, iX + 45, 1):
				for iYLoop in range(iY - 50, iY + 50, 1):
					pPlot = CyMap().plot(iXLoop, iYLoop)
					if ( pPlot.getTeam()!=pTID ):
						if ( pPlot.getTerrainType()==tt_desert ):
							if ( pPlot.getFeatureType()!=gc.getInfoTypeForString( 'FEATURE_FLOOD_PLAINS' )) and ( pPlot.getFeatureType()!=gc.getInfoTypeForString( 'FEATURE_OASIS' )) and ( pPlot.getFeatureType()!=gc.getInfoTypeForString( 'FEATURE_SCRUB' )):
								pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FALLOUT" ), 1)
						elif ( pPlot.getTerrainType()==tt_ocean ) or ( pPlot.getTerrainType()==tt_coast ):
							if ( pPlot.getFeatureType()==gc.getInfoTypeForString( 'FEATURE_ICE' )):
								pPlot.setFeatureType(-1, 0)
						elif ( pPlot.getTerrainType()==tt_snow ):
							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_TUNDRA" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_marsh ):
							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_PLAINS" ), 1, 1)
						elif ( pPlot.getTerrainType()==tt_plain ):
							if ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_FARM' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WINDMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_MINE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WATERMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WORKSHOP' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_LUMBERMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_PLANTATION' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_QUARRY' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_PASTURE' ))and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_CAMP' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WELL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WINERY' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_COTTAGE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_HAMLET' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_VILLAGE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_TOWN' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_FOREST_PRESERVE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_FORT' )):
								if ( pPlot.getFeatureType()!=gc.getInfoTypeForString( 'FEATURE_FOREST' )):
									pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_DESERT" ), 1, 1)
								if ( pPlot.getFeatureType()==gc.getInfoTypeForString( 'FEATURE_JUNGLE' )):
									pPlot.setFeatureType(-1, 0)
								if ( pPlot.getFeatureType()==gc.getInfoTypeForString( 'FEATURE_FOREST' )):
									pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_DEAD_FOREST" ), 1)
						elif ( pPlot.getTerrainType()==tt_grass ):
							if ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_FARM' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WINDMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_MINE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WATERMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WORKSHOP' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_LUMBERMILL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_PLANTATION' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_QUARRY' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_PASTURE' ))and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_CAMP' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WELL' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_WINERY' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_COTTAGE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_HAMLET' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_VILLAGE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_TOWN' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_FOREST_PRESERVE' )) and ( pPlot.getImprovementType()!=gc.getInfoTypeForString( 'IMPROVEMENT_FORT' )):
								if ( pPlot.getFeatureType()!=gc.getInfoTypeForString( 'FEATURE_FOREST' )):
									pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_PLAINS" ), 1, 1)
								if ( pPlot.getFeatureType()==gc.getInfoTypeForString( 'FEATURE_JUNGLE' )):
									pPlot.setFeatureType(-1, 0)
			
			CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_APOCALYPSE_PYTHON",()),'',0,',Art/Interface/Buttons/Units/Transport.dds,Art/Interface/Buttons/FinalFrontier1_Atlas.dds,6,14',ColorTypes(44), iX, iY, True,True)

		if iProjectType == gc.getInfoTypeForString('PROJECT_APOC'):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			plotX = pCity.getX()
			plotY = pCity.getY()

			earthquakeEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "EFFECT_ICBM_NUCLEAR_EXPLOSION" )
			#CyEngine( ).triggerEffect( earthquakeEffect, plot.getPoint( ) )
			CyEngine( ).triggerEffect( earthquakeEffect, gc.getMap( ).plot( pCity.getX() + 0, pCity.getY() + 0 ).getPoint( ) )

## Apocalypse End ##


		if ((pCity.getOwner() == gc.getGame().getActivePlayer())):
			popupInfo = CyPopupInfo()
			popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
			popupInfo.setData1(iProjectType)
			popupInfo.setData2(pCity.getID())
			popupInfo.setData3(2)
			popupInfo.setText(u"showWonderMovie")
			popupInfo.addPopup(pCity.getOwner())
	
	def onSelectionGroupPushMission(self, argsList):
		'selection group mission'
		eOwner = argsList[0]
		eMission = argsList[1]
		iNumUnits = argsList[2]
		listUnitIds = argsList[3]
		
		if (not self.__LOG_PUSH_MISSION):
			return
		if pHeadUnit:
			CvUtil.pyPrint("Selection Group pushed mission %d" %(eMission))
	
	def onUnitMove(self, argsList):
		'unit move'
		pPlot,pUnit,pOldPlot = argsList
		player = PyPlayer(pUnit.getOwner())
		unitInfo = PyInfo.UnitInfo(pUnit.getUnitType())
		if (not self.__LOG_MOVEMENT):
			return
		if player and unitInfo:
			CvUtil.pyPrint('Player %d Civilization %s unit %s is moving to %d, %d' 
				%(player.getID(), player.getCivilizationName(), unitInfo.getDescription(), 
				pUnit.getX(), pUnit.getY()))

	def onUnitSetXY(self, argsList):
		'units xy coords set manually'
		pPlot,pUnit = argsList
		player = PyPlayer(pUnit.getOwner())
		unitInfo = PyInfo.UnitInfo(pUnit.getUnitType())
		if (not self.__LOG_MOVEMENT):
			return
		
	def onUnitCreated(self, argsList):
		'Unit Completed'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())

## Sweden Start ##

		pPlayer = gc.getPlayer(unit.getOwner())

		if unit.getUnitType() == gc.getInfoTypeForString('UNIT_SWEDEN'):
			unit.changeExperience(3, -1, False, False, False)

## Sweden End ##

## Str Trait Start ##
#
#		iTrait = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_STRATEGIC')
#		
#		if (pPlayer.hasTrait(iTrait)):
#			unit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_COMBATV'), True)
#
## Str Trait End ##

		if (not self.__LOG_UNITBUILD):
			return

	def onUnitBuilt(self, argsList):
		'Unit Completed'
		city = argsList[0]
		unit = argsList[1]
		player = PyPlayer(city.getOwner())

## Sweden Start ##

		pPlayer = gc.getPlayer(unit.getOwner())

		if unit.getUnitType() == gc.getInfoTypeForString('UNIT_SWEDEN'):
			unit.changeExperience(3, -1, False, False, False)

## Sweden End ##

## Bar Trait Start ##

		iTrait = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_BARB')
		
		if (pPlayer.hasTrait(iTrait)):
			unit.changeExperience(1, -1, False, False, False)

## Bar Trait End ##

## Str Trait Start ##
#
#		iTrait = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_STRATEGIC')
#		
#		if (pPlayer.hasTrait(iTrait)):
#			unit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_COMBATV'), True)
#
## Str Trait End ##

## Silo Start ##

		if city.getNumRealBuilding(gc.getInfoTypeForString('BUILDING_SILO')) > 0:
			if (unit.getUnitType() == gc.getInfoTypeForString('UNIT_GUIDED_MISSILE')) or (unit.getUnitType() == gc.getInfoTypeForString('UNIT_TACTICAL_NUKE')) or (unit.getUnitType() == gc.getInfoTypeForString('UNIT_BIOLOGICAL_WARFARE_MISSILE')):
				unit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_RANGE3'), True)

## Silo End ##

## Sun Tzu Start ##

		if city.getNumRealBuilding(gc.getInfoTypeForString('BUILDING_SWAR')) > 0:
			#if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_BIOLOGICAL')) and (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_AIR')) and (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_DIPLOMAT')) and (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_MISSLE')) and (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_SPY')) and (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_WORKER')) and (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_SETTLER')) and (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_ANIMAL')) and (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_DOOM')):
			if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_DIPLOMAT')):
				if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_SPY')):
					if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_WORKER')):
						if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_SETTLER')):
							if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_AIR')):
								if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_MISSLE')):
									if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_BIOLOGICAL')):
										if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_DOOM')):
											if (unit.getUnitCombatType() != gc.getInfoTypeForString('UNITCOMBAT_ANIMAL')): 
												unit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_COMBAT10'), True)

## Sun Tzu End ##

		CvAdvisorUtils.unitBuiltFeats(city, unit)
		
		if (not self.__LOG_UNITBUILD):
			return
		CvUtil.pyPrint('%s was finished by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))
	
	def onUnitKilled(self, argsList):
		'Unit Killed'
		unit, iAttacker = argsList
		player = PyPlayer(unit.getOwner())
		attacker = PyPlayer(iAttacker)
		if (not self.__LOG_UNITKILLED):
			return
		CvUtil.pyPrint('Player %d Civilization %s Unit %s was killed by Player %d' 
			%(player.getID(), player.getCivilizationName(), PyInfo.UnitInfo(unit.getUnitType()).getDescription(), attacker.getID()))

	def onUnitLost(self, argsList):
		'Unit Lost'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())
		if (not self.__LOG_UNITLOST):
			return
		CvUtil.pyPrint('%s was lost by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))
	
	def onUnitPromoted(self, argsList):
		'Unit Promoted'
		pUnit, iPromotion = argsList
		player = PyPlayer(pUnit.getOwner())
		if (not self.__LOG_UNITPROMOTED):
			return
		CvUtil.pyPrint('Unit Promotion Event: %s - %s' %(player.getCivilizationName(), pUnit.getName(),))
	
	def onUnitSelected(self, argsList):
		'Unit Selected'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())
		if (not self.__LOG_UNITSELECTED):
			return
		CvUtil.pyPrint('%s was selected by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))
	
	def onUnitRename(self, argsList):
		'Unit is renamed'
		pUnit = argsList[0]
		if (pUnit.getOwner() == CyGame().getActivePlayer()):
			self.__eventEditUnitNameBegin(pUnit)
	
	def onUnitPillage(self, argsList):
		'Unit pillages a plot'
		pUnit, iImprovement, iRoute, iOwner = argsList
		iPlotX = pUnit.getX()
		iPlotY = pUnit.getY()
		pPlot = CyMap().plot(iPlotX, iPlotY)
		
		if (not self.__LOG_UNITPILLAGE):
			return
		CvUtil.pyPrint("Player %d's %s pillaged improvement %d and route %d at plot at (%d, %d)" 
			%(iOwner, PyInfo.UnitInfo(pUnit.getUnitType()).getDescription(), iImprovement, iRoute, iPlotX, iPlotY))
	
	def onUnitSpreadReligionAttempt(self, argsList):
		'Unit tries to spread religion to a city'
		pUnit, iReligion, bSuccess = argsList
		
		iX = pUnit.getX()
		iY = pUnit.getY()
		pPlot = CyMap().plot(iX, iY)
		pCity = pPlot.getPlotCity()
	
	def onUnitGifted(self, argsList):
		'Unit is gifted from one player to another'
		pUnit, iGiftingPlayer, pPlotLocation = argsList

## Str Trait Start ##
#
#		iStrategic = gc.getInfoTypeForString('PROMOTION_COMBATV')
#
#		if pUnit.isHasPromotion(iStrategic):
#			pUnit.setHasPromotion(iStrategic, False)
#
#		pPlayer = gc.getPlayer( pPlotLocation.getOwner( ) )
#		iTrait = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_STRATEGIC')
#		
#		if (pPlayer.hasTrait(iTrait)):
#			pUnit.setHasPromotion(iStrategic, True)
#
## Str Trait End ##
	
	def onUnitBuildImprovement(self, argsList):
		'Unit begins enacting a Build (building an Improvement or Route)'
		pUnit, iBuild, bFinished = argsList

	def onGoodyReceived(self, argsList):
		'Goody received'
		iPlayer, pPlot, pUnit, iGoodyType = argsList
		if (not self.__LOG_GOODYRECEIVED):
			return
		CvUtil.pyPrint('%s received a goody' %(gc.getPlayer(iPlayer).getCivilizationDescription(0)),)
	
	def onGreatPersonBorn(self, argsList):
		'Unit Promoted'
		pUnit, iPlayer, pCity = argsList
		player = PyPlayer(iPlayer)
		if pUnit.isNone() or pCity.isNone():
			return
		if (not self.__LOG_GREATPERSON):
			return
		CvUtil.pyPrint('A %s was born for %s in %s' %(pUnit.getName(), player.getCivilizationName(), pCity.getName()))
	
	def onTechAcquired(self, argsList):
		'Tech Acquired'
		iTechType, iTeam, iPlayer, bAnnounce = argsList
		# Note that iPlayer may be NULL (-1) and not a refer to a player object
		
## Technocracy Test ##
#
#		pPlayer = gc.getPlayer(iPlayer)
#		pTechType = gc.getTechInfo(iTechType)
#		lPlayers = PyGame().getCivPlayerList( )
#
#		iGovernmentCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_GOVERNMENT')
#		iTechnocracy = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_TECHNOCRACY')
#		iGovernmentCivic = pPlayer.getCivics(iGovernmentCivicOption)
#
#		for iPlayer in range( len( lPlayers ) ):
#			if (iGovernmentCivic == iTechnocracy):
#
#				lCities = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getCityList( )
#				for iCity in range( len( lCities ) ):
#					pCity = lCities[ iCity ]
#					pCity.changeCulture( 5 )
#
## Technocracy Test ##

		# Show tech splash when applicable
		if (iPlayer > -1 and bAnnounce and not CyInterface().noTechSplash()):
			if (gc.getGame().isFinalInitialized() and not gc.getGame().GetWorldBuilderMode()):
				if (iPlayer == CyGame().getActivePlayer()):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
					popupInfo.setData1(iTechType)
					popupInfo.setText(u"showTechSplash")
					popupInfo.addPopup(iPlayer)
				
		if (not self.__LOG_TECH):
			return
		CvUtil.pyPrint('%s was finished by Team %d' 
			%(PyInfo.TechnologyInfo(iTechType).getDescription(), iTeam))
	
	def onTechSelected(self, argsList):
		'Tech Selected'
		iTechType, iPlayer = argsList
		if (not self.__LOG_TECH):
			return
		CvUtil.pyPrint('%s was selected by Player %d' %(PyInfo.TechnologyInfo(iTechType).getDescription(), iPlayer))
	
	def onReligionFounded(self, argsList):
		'Religion Founded'
		iReligion, iFounder = argsList
		player = PyPlayer(iFounder)
		
		iCityId = gc.getGame().getHolyCity(iReligion).getID()
		if (gc.getGame().isFinalInitialized() and not gc.getGame().GetWorldBuilderMode()):
			if (iFounder == CyGame().getActivePlayer()):
				popupInfo = CyPopupInfo()
				popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
				popupInfo.setData1(iReligion)
				popupInfo.setData2(iCityId)
				popupInfo.setData3(1)
				popupInfo.setText(u"showWonderMovie")
				popupInfo.addPopup(iFounder)
		
		if (not self.__LOG_RELIGION):
			return
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getReligionInfo(iReligion).getDescription()))

	def onReligionSpread(self, argsList):
		'Religion Has Spread to a City'
		iReligion, iOwner, pSpreadCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return

		CvUtil.pyPrint('%s has spread to Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pSpreadCity.getName()))

	def onReligionRemove(self, argsList):
		'Religion Has been removed from a City'
		iReligion, iOwner, pRemoveCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has been removed from Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pRemoveCity.getName()))
				
	def onCorporationFounded(self, argsList):
		'Corporation Founded'
		iCorporation, iFounder = argsList
		player = PyPlayer(iFounder)
		
		if (not self.__LOG_RELIGION):
			return
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getCorporationInfo(iCorporation).getDescription()))

	def onCorporationSpread(self, argsList):
		'Corporation Has Spread to a City'
		iCorporation, iOwner, pSpreadCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has spread to Player %d Civilization %s city of %s'
			%(gc.getCorporationInfo(iCorporation).getDescription(), iOwner, player.getCivilizationName(), pSpreadCity.getName()))

	def onCorporationRemove(self, argsList):
		'Corporation Has been removed from a City'
		iCorporation, iOwner, pRemoveCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has been removed from Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pRemoveCity.getName()))
				
	def onGoldenAge(self, argsList):
		'Golden Age'
		iPlayer = argsList[0]
		player = PyPlayer(iPlayer)
		if (not self.__LOG_GOLDENAGE):
			return
		CvUtil.pyPrint('Player %d Civilization %s has begun a golden age'
			%(iPlayer, player.getCivilizationName()))

	def onEndGoldenAge(self, argsList):
		'End Golden Age'
		iPlayer = argsList[0]
		player = PyPlayer(iPlayer)
		if (not self.__LOG_ENDGOLDENAGE):
			return
		CvUtil.pyPrint('Player %d Civilization %s golden age has ended'
			%(iPlayer, player.getCivilizationName()))

	def onChangeWar(self, argsList):
		'War Status Changes'
		bIsWar = argsList[0]
		iTeam = argsList[1]
		iRivalTeam = argsList[2]
		if (not self.__LOG_WARPEACE):
			return
		if (bIsWar):
			strStatus = "declared war"
		else:
			strStatus = "declared peace"
		CvUtil.pyPrint('Team %d has %s on Team %d'
			%(iTeam, strStatus, iRivalTeam))
	
	def onChat(self, argsList):
		'Chat Message Event'
		chatMessage = "%s" %(argsList[0],)
		
	def onSetPlayerAlive(self, argsList):
		'Set Player Alive Event'
		iPlayerID = argsList[0]
		bNewValue = argsList[1]
		CvUtil.pyPrint("Player %d's alive status set to: %d" %(iPlayerID, int(bNewValue)))
		
	def onPlayerChangeStateReligion(self, argsList):
		'Player changes his state religion'
		iPlayer, iNewReligion, iOldReligion = argsList
		
	def onPlayerGoldTrade(self, argsList):
		'Player Trades gold to another player'
		iFromPlayer, iToPlayer, iGoldAmount = argsList
		
	def onCityBuilt(self, argsList):
		'City Built'
		city = argsList[0]

## Agr Trait Start ##

		player = PyPlayer(city.getOwner())

		pPlayer = gc.getPlayer(city.getOwner())

		iTrait = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_AGR')

		if (pPlayer.hasTrait(iTrait)):
			city.changePopulation(1)

## Agr Trait End ##

## Nat Trait Start ##

		NBuilding = "BUILDING_NAT_BONUS"

		iTrait2 = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_NAT')

		if (pPlayer.hasTrait(iTrait2)):
			city.setNumRealBuilding(gc.getInfoTypeForString(NBuilding), 1)

## Nat Trait End ##

		if (city.getOwner() == CyGame().getActivePlayer()):
			self.__eventEditCityNameBegin(city, False)	
		CvUtil.pyPrint('City Built Event: %s' %(city.getName()))
		
	def onCityRazed(self, argsList):
		'City Razed'
		city, iPlayer = argsList
		iOwner = city.findHighestCulture()

# Partisans! Start #

		# Partisans!
		if city.getPopulation > 1 and iOwner != -1 and iPlayer != -1:
			owner = gc.getPlayer(iOwner)
			if not owner.isBarbarian() and owner.getNumCities() > 0:
				if gc.getTeam(owner.getTeam()).isAtWar(gc.getPlayer(iPlayer).getTeam()):
					if gc.getNumEventTriggerInfos() > 0: # prevents mods that don't have events from getting an error
						iEvent = CvUtil.findInfoTypeNum(gc.getEventTriggerInfo, gc.getNumEventTriggerInfos(),'EVENTTRIGGER_PARTISANS')
						if iEvent != -1 and gc.getGame().isEventActive(iEvent) and owner.getEventTriggerWeight(iEvent) < 0:
							triggerData = owner.initTriggeredData(iEvent, true, -1, city.getX(), city.getY(), iPlayer, city.getID(), -1, -1, -1, -1)
			
		
		owner = PyPlayer(city.getOwner())
		razor = PyPlayer(iPlayer)

# Partisans! End #

# Next War Start #
		
		self.iArcologyCityID = -1
		
		if city.getNumRealBuilding(gc.getInfoTypeForString("BUILDING_ARCOLOGY")) or city.getNumRealBuilding(gc.getInfoTypeForString("BUILDING_ARCOLOGY_SHIELDING")) or city.getNumRealBuilding(gc.getInfoTypeForString("BUILDING_ADVANCED_SHIELDING")):
			self.iArcologyCityID = city.getID()

# Next War End #
		
		CvUtil.pyPrint('Player %d Civilization %s City %s was razed by Player %d' 
			%(owner.getID(), owner.getCivilizationName(), city.getName(), razor.getID()))
		CvUtil.pyPrint("City Razed Event: %s" %(city.getName(),))
	
	def onCityAcquired(self, argsList):
		'City Acquired'
		iPreviousOwner,iNewOwner,pCity,bConquest,bTrade = argsList
		CvUtil.pyPrint('City Acquired Event: %s' %(pCity.getName()))

# Nat Trait Start #

		pPlayer = gc.getPlayer(iNewOwner)
		iX = pCity.getX()
		iY = pCity.getY()
		pPlot = CyMap().plot(iX +0, iY +0)

		iTrait = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_NAT')
		iBuildingMagLev = CvUtil.findInfoTypeNum(gc.getBuildingInfo,gc.getNumBuildingInfos(),'BUILDING_NAT_BONUS')
		iAqueduct = CvUtil.findInfoTypeNum(gc.getBuildingInfo,gc.getNumBuildingInfos(),'BUILDING_AQUEDUCT')
		iBerber = CvUtil.findInfoTypeNum(gc.getBuildingInfo,gc.getNumBuildingInfos(),'BUILDING_BERBER')


		if (pPlayer.hasTrait(iTrait)):
			pCity.setNumRealBuilding(iBuildingMagLev, 1)
		else:
			if pCity.getNumActiveBuilding(iBuildingMagLev) == true:
				pCity.setNumRealBuilding(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), "BUILDING_NAT_BONUS"), False)

# Nat Trait End #

# Great Bath Start #

		b_bath = gc.getInfoTypeForString("BUILDING_GREAT_BATH")

		if pCity.getNumActiveBuilding(b_bath) == true:
			pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_RAINWATER_BASIN" ), 1)

# Great Bath End #

# Berber Start #

		if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_BERBER")):
			if (pCity.getNumActiveBuilding(iAqueduct) == true) or (pCity.getNumActiveBuilding(iBerber) == true):
				pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_FOSSIL_WATER" ), 1)

# Berber End #

# Flavian Start #

		### is building obsolete? ###
		pNewOwner = gc.getPlayer(iNewOwner)
		b_BUILDING_PETRA = gc.getInfoTypeForString("BUILDING_FLAV")
		obsoleteTech = gc.getBuildingInfo(b_BUILDING_PETRA).getObsoleteTech()
		if ( gc.getTeam(pNewOwner.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):

			### search for Petra ###
			petra = false
			for iCity in range(pNewOwner.getNumCities()):
				if petra == false:
					ppCity = pNewOwner.getCity(iCity)
					if ( ppCity.getNumActiveBuilding(b_BUILDING_PETRA)==true ):
						petra = true

			if petra == true:

				### find UB and DB ###
				civ_type = gc.getPlayer(iPreviousOwner).getCivilizationType()
				for iBuilding in range(gc.getNumBuildingClassInfos()):
					iUniqueBuilding = gc.getCivilizationInfo(civ_type).getCivilizationBuildings(iBuilding);
					iDefaultBuilding = gc.getBuildingClassInfo(iBuilding).getDefaultBuildingIndex();
					if (iDefaultBuilding > -1 and iUniqueBuilding > -1 and iDefaultBuilding != iUniqueBuilding):

						if ( pCity.getNumActiveBuilding(iDefaultBuilding)==true ):

							### replace DB with UB ###
							pCity.setNumRealBuilding(iDefaultBuilding,0)
							pCity.setNumRealBuilding(iUniqueBuilding,1)

							### ausgabe ###
							CyInterface().addMessage(iNewOwner,false,15,CyTranslator().getText("TXT_KEY_PETRA_GAMETXT1",( pCity.getName(), PyHelpers.PyInfo.BuildingInfo(iUniqueBuilding).getDescription() , PyHelpers.PyInfo.BuildingInfo(iDefaultBuilding).getDescription() )),'',0,gc.getBuildingInfo(iUniqueBuilding).getButton(),ColorTypes(11),pCity.getX(),pCity.getY(),True,True)
							### message: We captured a %s2 in the City of %s1! We will use it instead of the %s3. ###

## Flavian End ##
	
	def onCityAcquiredAndKept(self, argsList):
		'City Acquired and Kept'
		iOwner,pCity = argsList
		CvUtil.pyPrint('City Acquired and Kept Event: %s' %(pCity.getName()))
	
	def onCityLost(self, argsList):
		'City Lost'
		city = argsList[0]
		player = PyPlayer(city.getOwner())

# Berber Start #

		iX = city.getX()
		iY = city.getY()
		pPlot = CyMap().plot(iX +0, iY +0)

		if ( pPlot.getFeatureType()==gc.getInfoTypeForString( 'FEATURE_FOSSIL_WATER' )):
			pPlot.setFeatureType(-1, 0)

# Berber End #

# Great Bath Start #

		if ( pPlot.getFeatureType()==gc.getInfoTypeForString( 'FEATURE_RAINWATER_BASIN' )):
			pPlot.setFeatureType(-1, 0)

# Great Bath End #

# Next War Start #
		
		if city.getID() == self.iArcologyCityID:
			city.plot().setImprovementType(gc.getInfoTypeForString("IMPROVEMENT_CITY_RUINS_ARCOLOGY"))

# Next War End #

# Nat Trait Start #
#		
#		NBuilding = "BUILDING_NAT_BONUS"
#
#		if (city.isHasBuilding(NBuilding)):
#			city.setNumRealBuilding(gc.getInfoTypeForString(NBuilding), -1)
#
# Nat Trait End #
		
		if (not self.__LOG_CITYLOST):
			return
		CvUtil.pyPrint('City %s was lost by Player %d Civilization %s' 
			%(city.getName(), player.getID(), player.getCivilizationName()))
	
	def onCultureExpansion(self, argsList):
		'City Culture Expansion'
		pCity = argsList[0]
		iPlayer = argsList[1]
		CvUtil.pyPrint("City %s's culture has expanded" %(pCity.getName(),))
	
	def onCityGrowth(self, argsList):
		'City Population Growth'
		pCity = argsList[0]
		iPlayer = argsList[1]

## Agr Trait Start ##

		player = PyPlayer(pCity.getOwner())

		pPlayer = gc.getPlayer(pCity.getOwner())

		iTrait = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_AGR')

		if (pPlayer.hasTrait(iTrait)):

			estiEnd = CyGame().getEstimateEndTurn()
			if ( estiEnd >= 1000 ):
				pCity.changeFood( +16 )
			elif ( estiEnd >= 700 ):
				pCity.changeFood( +8 )
			elif ( estiEnd >= 500 ):
				pCity.changeFood( +6 )
			elif ( estiEnd >= 300 ):
				pCity.changeFood( +4 )

## Agr Trait End ##

## Independent Farmers Start ##

		iLaborCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_LABOR')
		iFarmers = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_INDEPENDENT_FARMERS')
		iLaborCivic = pPlayer.getCivics(iLaborCivicOption)

		if (iLaborCivic == iFarmers):

			estiEnd = CyGame().getEstimateEndTurn()
			if ( estiEnd >= 1000 ):
				pCity.changeFood( +12 )
			elif ( estiEnd >= 700 ):
				pCity.changeFood( +6 )
			elif ( estiEnd >= 500 ):
				pCity.changeFood( +4 )
			elif ( estiEnd >= 300 ):
				pCity.changeFood( +2 )

## Independent Farmers Start ##

		CvUtil.pyPrint("%s has grown" %(pCity.getName(),))
	
	def onCityDoTurn(self, argsList):
		'City Production'
		pCity = argsList[0]
		iPlayer = argsList[1]

		CvAdvisorUtils.cityAdvise(pCity, iPlayer)
	
	def onCityBuildingUnit(self, argsList):
		'City begins building a unit'
		pCity = argsList[0]
		iUnitType = argsList[1]
		if (not self.__LOG_CITYBUILDING):
			return
		CvUtil.pyPrint("%s has begun building a %s" %(pCity.getName(),gc.getUnitInfo(iUnitType).getDescription()))
	
	def onCityBuildingBuilding(self, argsList):
		'City begins building a Building'
		pCity = argsList[0]
		iBuildingType = argsList[1]
		if (not self.__LOG_CITYBUILDING):
			return
		CvUtil.pyPrint("%s has begun building a %s" %(pCity.getName(),gc.getBuildingInfo(iBuildingType).getDescription()))
	
	def onCityRename(self, argsList):
		'City is renamed'
		pCity = argsList[0]
		if (pCity.getOwner() == CyGame().getActivePlayer()):
			self.__eventEditCityNameBegin(pCity, True)	
	
	def onCityHurry(self, argsList):
		'City is renamed'
		pCity = argsList[0]
		iHurryType = argsList[1]

## Templo Mayor Start ##

		if iHurryType == gc.getInfoTypeForString( "HURRY_POPULATION" ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			b_Templo_Mayor = gc.getInfoTypeForString("BUILDING_TEMPLO_MAYOR")
			obsoleteTech = gc.getBuildingInfo(b_Templo_Mayor).getObsoleteTech()
			if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):

				### check all player cities... ###
				for iCity in range(pPlayer.getNumCities()):
					ppCity = pPlayer.getCity(iCity)

					### ...if he has the Templo Mayor Wonder ###
					if ppCity.getNumActiveBuilding(b_Templo_Mayor) == true:
						### reset hurry anger in hurry city ###
						HAT = pCity.getHurryAngerTimer()
						pCity.changeHurryAngerTimer(-HAT / 2)

						### ausgabe ###
						CyInterface().addMessage(CyGame().getActivePlayer(),false,15,CyTranslator().getText("TXT_KEY_TEMPLO_MAYOR_GAMETXT1",(pCity.getName(),ppCity.getName())),'',0,gc.getBuildingInfo(b_Templo_Mayor).getButton(),ColorTypes(44), pCity.getX(), pCity.getY(), True,True)
						### message: Due to the Templo Mayor Wonder the city of %s didn't suffer any hurry anger! ###

						### add temp happiness in wonder city ###
						HAT2 = HAT / 2
						HAT2 = int(HAT2)
						HT = ppCity.getHappinessTimer()
						ppCity.changeHappinessTimer(HAT2)

						### ausgabe ###
						CyInterface().addMessage(CyGame().getActivePlayer(),false,15,CyTranslator().getText("TXT_KEY_TEMPLO_MAYOR_GAMETXT2",(pCity.getName(),ppCity.getName())),'',0,gc.getBuildingInfo(b_Templo_Mayor).getButton(),ColorTypes(44), ppCity.getX(), ppCity.getY(), True,True)
						### message: The sacrifice of our people in %s brings some happines in the city of %s! ###

## Templo Mayor End ##

## Sydney Opera House Start ##

		if iHurryType == gc.getInfoTypeForString( "HURRY_GOLD" ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			b_Sydney = gc.getInfoTypeForString("BUILDING_SYDNEY_OPERA_HOUSE")
			obsoleteTech = gc.getBuildingInfo(b_Sydney).getObsoleteTech()
			pPID = pPlayer.getID()

			if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
				for iCity in range(pPlayer.getNumCities()):
					ppCity = pPlayer.getCity(iCity)
					if ppCity.getNumActiveBuilding(b_Sydney) == true:

						iRelCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_RELIGION')
						iPacifist = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_PACIFISM')
						iRelCivic = pPlayer.getCivics(iRelCivicOption)
						iPhi = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_PHILOSOPHICAL')
						iNational = CvUtil.findInfoTypeNum(gc.getBuildingInfo,gc.getNumBuildingInfos(),'BUILDING_NATIONAL_EPIC')

						if (iRelCivic == iPacifist):
							if (pPlayer.hasTrait(iPhi)):
								if pCity.getNumActiveBuilding(iNational) == true:
									pCity.changeGreatPeopleProgress( +12 )
								elif pCity.getNumActiveBuilding(iNational) == false:
									pCity.changeGreatPeopleProgress( +9 )
							else:
								if pCity.getNumActiveBuilding(iNational) == true:
									pCity.changeGreatPeopleProgress( +9 )
								elif pCity.getNumActiveBuilding(iNational) == false:
									pCity.changeGreatPeopleProgress( +6 )
						elif (iRelCivic != iPacifist):
							if (pPlayer.hasTrait(iPhi)):
								if pCity.getNumActiveBuilding(iNational) == true:
									pCity.changeGreatPeopleProgress( +9 )
								elif pCity.getNumActiveBuilding(iNational) == false:
									pCity.changeGreatPeopleProgress( +6 )
							else:
								if pCity.getNumActiveBuilding(iNational) == true:
									pCity.changeGreatPeopleProgress( +6 )
								elif pCity.getNumActiveBuilding(iNational) == false:
									pCity.changeGreatPeopleProgress( +3 )
						else:
							pCity.changeGreatPeopleProgress( +3 )
							
## Sydney Opera House End ##

## Tupi Start ##

		if iHurryType == gc.getInfoTypeForString( "HURRY_POPULATION" ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			b_Tupi = gc.getInfoTypeForString("BUILDING_TUPI")
			obsoleteTech = gc.getBuildingInfo(b_Tupi).getObsoleteTech()
			if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
				## for iCity in range(pPlayer.getNumCities()):
				## ppCity = pPlayer.getCity(iCity) 
				if pCity.getNumActiveBuilding(b_Tupi) == true:
					estiEnd = CyGame().getEstimateEndTurn()
					if ( estiEnd >= 1000 ):
						pCity.changeFood( +16 )
					elif ( estiEnd >= 700 ):
						pCity.changeFood( +8 )
					elif ( estiEnd >= 500 ):
						pCity.changeFood( +6 )
					elif ( estiEnd >= 300 ):
						pCity.changeFood( +4 )


## Tupi End ##

## Austria Start ##

		if iHurryType == gc.getInfoTypeForString( "HURRY_GOLD" ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			b_Austria = gc.getInfoTypeForString("BUILDING_AUSTRIA_OPERAHOUSE")
			obsoleteTech = gc.getBuildingInfo(b_Austria).getObsoleteTech()
			if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
				## for iCity in range(pPlayer.getNumCities()):
				## ppCity = pPlayer.getCity(iCity)
				if pCity.getNumActiveBuilding(b_Austria) == true:
					pCC = pCity.getCulture(pPID)
					pCC = int( + 15)
					pCity.changeCulture(pPID, pCC, true)

## Austria End ##

## Kongo Start ##

		if iHurryType == gc.getInfoTypeForString( "HURRY_POPULATION" ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			b_Tupi = gc.getInfoTypeForString("BUILDING_CONGO")
			obsoleteTech = gc.getBuildingInfo(b_Tupi).getObsoleteTech()
			if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
				## for iCity in range(pPlayer.getNumCities()):
				## ppCity = pPlayer.getCity(iCity)
				if pCity.getNumActiveBuilding(b_Tupi) == true:
					iGold = pPlayer.getGold( )
					## if pPlayer.getGold( ) > 0:
					## pPlayer.changeGold( iGold//20 )
					## else:
					estiEnd = CyGame().getEstimateEndTurn()
					if ( estiEnd >= 1000 ):
						pPlayer.changeGold( +50 )
					elif ( estiEnd >= 700 ):
						pPlayer.changeGold( +25 )
					elif ( estiEnd >= 500 ):
						pPlayer.changeGold( +20 )
					elif ( estiEnd >= 300 ):
						pPlayer.changeGold( +15 )


## Kongo End ##

	def onVictory(self, argsList):
		'Victory'
		iTeam, iVictory = argsList
		if (iVictory >= 0 and iVictory < gc.getNumVictoryInfos()):
			victoryInfo = gc.getVictoryInfo(int(iVictory))
			CvUtil.pyPrint("Victory!  Team %d achieves a %s victory"
				%(iTeam, victoryInfo.getDescription()))
	
	def onVassalState(self, argsList):
		'Vassal State'
		iMaster, iVassal, bVassal = argsList
		
		if (bVassal):
			CvUtil.pyPrint("Team %d becomes a Vassal State of Team %d"
				%(iVassal, iMaster))
		else:
			CvUtil.pyPrint("Team %d revolts and is no longer a Vassal State of Team %d"
				%(iVassal, iMaster))
	
	def onGameUpdate(self, argsList):
		'sample generic event, called on each game turn slice'
		genericArgs = argsList[0][0]	# tuple of tuple of my args
		turnSlice = genericArgs[0]
	
	def onMouseEvent(self, argsList):
		'mouse handler - returns 1 if the event was consumed'
		eventType,mx,my,px,py,interfaceConsumed,screens = argsList
		if ( px!=-1 and py!=-1 ):
			if ( eventType == self.EventLButtonDown ):
				if (self.bAllowCheats and self.bCtrl and self.bAlt and CyMap().plot(px,py).isCity() and not interfaceConsumed):
					# Launch Edit City Event
					self.beginEvent( CvUtil.EventEditCity, (px,py) )
					return 1
				
				elif (self.bAllowCheats and self.bCtrl and self.bShift and not interfaceConsumed):
					# Launch Place Object Event
					self.beginEvent( CvUtil.EventPlaceObject, (px, py) )
					return 1
			
		if ( eventType == self.EventBack ):
			return CvScreensInterface.handleBack(screens)
		elif ( eventType == self.EventForward ):
			return CvScreensInterface.handleForward(screens)
		
		return 0
		

#################### TRIGGERED EVENTS ##################	
	
	def __eventEditCityNameBegin(self, city, bRename):
		popup = PyPopup.PyPopup(CvUtil.EventEditCityName, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setUserData((city.getID(), bRename))
		popup.setHeaderString(localText.getText("TXT_KEY_NAME_CITY", ()))
		popup.setBodyString(localText.getText("TXT_KEY_SETTLE_NEW_CITY_NAME", ()))
		popup.createEditBox(city.getName())
		popup.setEditBoxMaxCharCount( 15 )
		popup.launch()
	
	def __eventEditCityNameApply(self, playerID, userData, popupReturn):	
		'Edit City Name Event'
		iCityID = userData[0]
		bRename = userData[1]
		player = gc.getPlayer(playerID)
		city = player.getCity(iCityID)
		cityName = popupReturn.getEditBoxString(0)
		if (len(cityName) > 30):
			cityName = cityName[:30]
		city.setName(cityName, not bRename)

	def __eventEditCityBegin(self, argsList):
		'Edit City Event'
		px,py = argsList
		CvWBPopups.CvWBPopups().initEditCity(argsList)
	
	def __eventEditCityApply(self, playerID, userData, popupReturn):
		'Edit City Event Apply'
		if (getChtLvl() > 0):
			CvWBPopups.CvWBPopups().applyEditCity( (popupReturn, userData) )

	def __eventPlaceObjectBegin(self, argsList):
		'Place Object Event'
		CvDebugTools.CvDebugTools().initUnitPicker(argsList)
	
	def __eventPlaceObjectApply(self, playerID, userData, popupReturn):
		'Place Object Event Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyUnitPicker( (popupReturn, userData) )

	def __eventAwardTechsAndGoldBegin(self, argsList):
		'Award Techs & Gold Event'
		CvDebugTools.CvDebugTools().cheatTechs()
	
	def __eventAwardTechsAndGoldApply(self, playerID, netUserData, popupReturn):
		'Award Techs & Gold Event Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyTechCheat( (popupReturn) )
	
	def __eventEditUnitNameBegin(self, argsList):
		pUnit = argsList
		popup = PyPopup.PyPopup(CvUtil.EventEditUnitName, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setUserData((pUnit.getID(),))
		popup.setBodyString(localText.getText("TXT_KEY_RENAME_UNIT", ()))
		popup.createEditBox(pUnit.getNameNoDesc())
		popup.launch()

	def __eventEditUnitNameApply(self, playerID, userData, popupReturn):	
		'Edit Unit Name Event'
		iUnitID = userData[0]
		unit = gc.getPlayer(playerID).getUnit(iUnitID)
		newName = popupReturn.getEditBoxString(0)
		if (len(newName) > 25):
			newName = newName[:25]			
		unit.setName(newName)

	def __eventWBAllPlotsPopupBegin(self, argsList):
		CvScreensInterface.getWorldBuilderScreen().allPlotsCB()
		return
	def __eventWBAllPlotsPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getButtonClicked() >= 0):
			CvScreensInterface.getWorldBuilderScreen().handleAllPlotsCB(popupReturn)
		return

	def __eventWBLandmarkPopupBegin(self, argsList):
		CvScreensInterface.getWorldBuilderScreen().setLandmarkCB("")
		#popup = PyPopup.PyPopup(CvUtil.EventWBLandmarkPopup, EventContextTypes.EVENTCONTEXT_ALL)
		#popup.createEditBox(localText.getText("TXT_KEY_WB_LANDMARK_START", ()))
		#popup.launch()
		return

	def __eventWBLandmarkPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getEditBoxString(0)):
			szLandmark = popupReturn.getEditBoxString(0)
			if (len(szLandmark)):
				CvScreensInterface.getWorldBuilderScreen().setLandmarkCB(szLandmark)
		return

	def __eventWBScriptPopupBegin(self, argsList):
		popup = PyPopup.PyPopup(CvUtil.EventWBScriptPopup, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setHeaderString(localText.getText("TXT_KEY_WB_SCRIPT", ()))
		popup.createEditBox(CvScreensInterface.getWorldBuilderScreen().getCurrentScript())
		popup.launch()
		return

	def __eventWBScriptPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getEditBoxString(0)):
			szScriptName = popupReturn.getEditBoxString(0)
			CvScreensInterface.getWorldBuilderScreen().setScriptCB(szScriptName)
		return

	def __eventWBStartYearPopupBegin(self, argsList):
		popup = PyPopup.PyPopup(CvUtil.EventWBStartYearPopup, EventContextTypes.EVENTCONTEXT_ALL)
		popup.createSpinBox(0, "", gc.getGame().getStartYear(), 1, 5000, -5000)
		popup.launch()
		return

	def __eventWBStartYearPopupApply(self, playerID, userData, popupReturn):
		iStartYear = popupReturn.getSpinnerWidgetValue(int(0))
		CvScreensInterface.getWorldBuilderScreen().setStartYearCB(iStartYear)
		return

# Social Reform
	def doEventEarthquake(self, plot):
		iSeverity = 3
		plotX = plot.getX()
		plotY = plot.getY()
		
		earthquakeEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "EFFECT_GREATPEOPLE_ACTIVE_PROPHET" )
		
		CyEngine( ).triggerEffect( earthquakeEffect, plot.getPoint( ) )
		CyEngine( ).triggerEffect( earthquakeEffect, gc.getMap( ).plot( plot.getX( ) + 1, plot.getY( ) + 1 ).getPoint( ) )
		CyEngine( ).triggerEffect( earthquakeEffect, gc.getMap( ).plot( plot.getX( ) + 1, plot.getY( ) - 1 ).getPoint( ) )
		CyEngine( ).triggerEffect( earthquakeEffect, gc.getMap( ).plot( plot.getX( ) - 1, plot.getY( ) + 1 ).getPoint( ) )		
		CyEngine( ).triggerEffect( earthquakeEffect, gc.getMap( ).plot( plot.getX( ) - 1, plot.getY( ) - 1 ).getPoint( ) )		
		
			
		for i in range(iSeverity):
			if i == 0:
				self.doEarthquakeDevastation(plotX, plotY, iSeverity)
			else:
				self.doEarthquakeDevastation(plotX - i, plotY, iSeverity-i)
				self.doEarthquakeDevastation(plotX - i, plotY - i, iSeverity-i)
				self.doEarthquakeDevastation(plotX, plotY - i, iSeverity-i)
				self.doEarthquakeDevastation(plotX + i, plotY - i, iSeverity-i)
				self.doEarthquakeDevastation(plotX + i, plotY, iSeverity-i)
				self.doEarthquakeDevastation(plotX + i, plotY + i, iSeverity-i)
				self.doEarthquakeDevastation(plotX, plotY + i, iSeverity-i)
				self.doEarthquakeDevastation(plotX - i, plotY + i, iSeverity-i)
	
	def doEarthquakeDevastation(self, iX, iY, iSeverity):
		plot = gc.getMap().plot(iX, iY)
		iEarthquakeEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "EFFECT_GREATPEOPLE_ACTIVE_PROPHET" )
		
		if plot.isCity():
			city = plot.getPlotCity()
			szCityName = city.getName()
			pyCity = PyCity(city.getOwner(), city.getID())
			if iSeverity < 100:
				iPopulation = city.getPopulation()
				if iPopulation < 100:
					iChange = +3
					city.changePopulation(iChange)
					if iChange > 0:
						szTitle = localText.getText("TXT_KEY_GODS_EARTHQUAKE_CITY_TITLE_CHANGED", (szCityName, ))
						CyInterface().addMessage(plot.getOwner(), True, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_WELOVEKING", InterfaceMessageTypes.MESSAGE_TYPE_INFO, gc.getEffectInfo(iEarthquakeEffect).getButton(), gc.getInfoTypeForString("COLOR_GREEN"), plot.getX(), plot.getY(), True, True)	

		if plot.getImprovementType() != -1:
			if self.getRandomNumber(100-iSeverity) >= 0:
				iCottage = CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_COTTAGE")
				iHamlet = CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_HAMLET")
				iVillage = CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_VILLAGE")
				iTown = CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_TOWN")

				ifort = CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_FORT")
				ifort2 = CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_FORT2")
				iseatown = CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_SUBMERGED_TOWN")
				iseatown2 = CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_SUBMERGED_TOWN2")
				
				szCottage = gc.getImprovementInfo(iCottage).getDescription()
				szHamlet = gc.getImprovementInfo(iHamlet).getDescription()
				szVillage = gc.getImprovementInfo(iVillage).getDescription()
				szTown = gc.getImprovementInfo(iTown).getDescription()

				szfort = gc.getImprovementInfo(ifort).getDescription()
				szfort2 = gc.getImprovementInfo(ifort2).getDescription()
				szseatown = gc.getImprovementInfo(iseatown).getDescription()
				szseatown2 = gc.getImprovementInfo(iseatown2).getDescription()
	
				cottageButton = gc.getImprovementInfo(iCottage).getButton()
				hamletButton = gc.getImprovementInfo(iHamlet).getButton()
				villageButton = gc.getImprovementInfo(iVillage).getButton()
				townButton = gc.getImprovementInfo(iTown).getButton()

				fortButton = gc.getImprovementInfo(ifort).getButton()
				fort2Button = gc.getImprovementInfo(ifort2).getButton()
				seatownButton = gc.getImprovementInfo(iseatown).getButton()
				seatown2Button = gc.getImprovementInfo(iseatown2).getButton()
							
				if plot.getImprovementType() == iCottage:
					plot.setImprovementType(iHamlet)
					szTitle = localText.getText("TXT_KEY_GODS_EARTHQUAKE_TOWN_REDUCTION_TITLE_CHANGED", (szCottage, szHamlet, ))
					CyInterface().addMessage(plot.getOwner(), False, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_IF_CHECKBOX", InterfaceMessageTypes.MESSAGE_TYPE_INFO, hamletButton, gc.getInfoTypeForString("COLOR_GREEN"), plot.getX(), plot.getY(), True, True)	
				elif plot.getImprovementType() == iHamlet:
					plot.setImprovementType(iVillage)
					szTitle = localText.getText("TXT_KEY_GODS_EARTHQUAKE_TOWN_REDUCTION_TITLE_CHANGED", (szHamlet, szVillage, ))
					CyInterface().addMessage(plot.getOwner(), False, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_IF_CHECKBOX", InterfaceMessageTypes.MESSAGE_TYPE_INFO, villageButton, gc.getInfoTypeForString("COLOR_GREEN"), plot.getX(), plot.getY(), True, True)	
				elif plot.getImprovementType() == CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_VILLAGE"):
					plot.setImprovementType(iTown)
					szTitle = localText.getText("TXT_KEY_GODS_EARTHQUAKE_TOWN_REDUCTION_TITLE_CHANGED", (szVillage, szTown, ))
					CyInterface().addMessage(plot.getOwner(), False, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_IF_CHECKBOX", InterfaceMessageTypes.MESSAGE_TYPE_INFO, townButton, gc.getInfoTypeForString("COLOR_GREEN"), plot.getX(), plot.getY(), True, True)

				elif plot.getImprovementType() == CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_FORT"):
					plot.setImprovementType(ifort2)
					szTitle = localText.getText("TXT_KEY_GODS_EARTHQUAKE_TOWN_REDUCTION_TITLE_CHANGED", (szfort, szfort2, ))
					CyInterface().addMessage(plot.getOwner(), False, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_IF_CHECKBOX", InterfaceMessageTypes.MESSAGE_TYPE_INFO, townButton, gc.getInfoTypeForString("COLOR_GREEN"), plot.getX(), plot.getY(), True, True)

				elif plot.getImprovementType() == CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), "IMPROVEMENT_SUBMERGED_TOWN"):
					plot.setImprovementType(iseatown2)
					szTitle = localText.getText("TXT_KEY_GODS_EARTHQUAKE_TOWN_REDUCTION_TITLE_CHANGED", (szseatown, szseatown2, ))
					CyInterface().addMessage(plot.getOwner(), False, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_IF_CHECKBOX", InterfaceMessageTypes.MESSAGE_TYPE_INFO, townButton, gc.getInfoTypeForString("COLOR_GREEN"), plot.getX(), plot.getY(), True, True)

# Diplomacy
	def doEventMinorMeteorStrike(self, centerPlot):
		plotX = centerPlot.getX()
		plotY = centerPlot.getY()

		plotPoint = gc.getMap().plot( plotX, plotY ).getPoint( )
		iMeteorEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "FIREWORKS_YELLOW_LARGE_SLOW" )
		CyEngine().triggerEffect( iMeteorEffect, plotPoint )

# Super Virus
	def doEventBlight( self, targetPlot ):
		plotX = targetPlot.getX()
		plotY = targetPlot.getY()
		iSeverity = 3

		plotPoint = gc.getMap().plot( plotX, plotY ).getPoint( )
		iBlightEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "EFFECT_GODS_BLIGHT" )
		CyEngine().triggerEffect( iBlightEffect, plotPoint )
		CyEngine().triggerEffect( iBlightEffect, plotPoint )

		
		for i in range(iSeverity):
			if i == 0:
				self.doBlightDevastation( plotX, plotY )
			else:
				self.doBlightDevastation( plotX - i, plotY )
				self.doBlightDevastation( plotX - i, plotY - i )
				self.doBlightDevastation( plotX, plotY - i )
				self.doBlightDevastation( plotX + i, plotY - i )
				self.doBlightDevastation( plotX + i, plotY )
				self.doBlightDevastation( plotX + i, plotY + i )
				self.doBlightDevastation( plotX, plotY + i )
				self.doBlightDevastation( plotX - i, plotY + i )

	def doBlightDevastation( self, plotX, plotY ):
			
		plot = gc.getMap().plot( plotX, plotY )

#Melts Ice
		if plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_OCEAN") or plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_COAST"):
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_ICE"):
				plot.setFeatureType(-1, 0)

#Coast
		if plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_COAST"):
			plot.setTerrainType(CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_COAST"), 1, 1)	

#Destroys Ocean Resources
		if plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_OCEAN"):
			plot.setTerrainType(CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_OCEAN"), 1, 1)	
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_FISH"):
				self.doBlightBonus( plot )				
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_CLAM"):
				self.doBlightBonus( plot )	
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_CRAB"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_WHALE"):
				self.doBlightBonus( plot )

#Degrades Plains, Destroys Plants & Animals		
		if plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_PLAINS"):
			plot.setTerrainType(CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_DESERT"), 1, 1)
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_RICE"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_WHEAT"):
				self.doBlightBonus( plot )				
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_WINE"):
				self.doBlightBonus( plot )				
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_SUGAR"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_HORSE"):
				self.doBlightBonus( plot )			
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_CORN"):
				self.doBlightBonus( plot )			
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_COW"):
				self.doBlightBonus( plot )				
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_DEER"):
				self.doBlightBonus( plot )	
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_PIG"):
				self.doBlightBonus( plot )			
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_SHEEP"):
				self.doBlightBonus( plot )	
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_FUR"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_COTTON"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_TOBACCO"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_BANANA"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_DYE"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_IVORY"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_SILK"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_SPICES"):
				self.doBlightBonus( plot )
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_FOREST"):
				self.doBlightFeature( plot )
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_JUNGLE"):
				self.doBlightFeature( plot )
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_OBSIDIAN_PLAINS"):
				self.doBlightFeature( plot )

#Degrades Grasslands, Destroys Animals	
		if plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_GRASS"):
			plot.setTerrainType(CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_PLAINS"), 1, 1)	
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_HORSE"):
				self.doBlightBonus( plot )				
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_COW"):
				self.doBlightBonus( plot )	
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_DEER"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_FUR"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_IVORY"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_PIG"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_SHEEP"):
				self.doBlightBonus( plot )
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_JUNGLE"):
				self.doBlightFeature( plot )
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_OBSIDIAN_PLAINS"):
				self.doBlightFeature( plot )

#Destroys any resource left on a Desert tile
		if plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_DESERT"):
			plot.setTerrainType(CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_DESERT"), 1, 1)	
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_INCENSE"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_RICE"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_WHEAT"):
				self.doBlightBonus( plot )				
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_WINE"):
				self.doBlightBonus( plot )				
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_SUGAR"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_HORSE"):
				self.doBlightBonus( plot )			
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_CORN"):
				self.doBlightBonus( plot )			
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_COW"):
				self.doBlightBonus( plot )				
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_DEER"):
				self.doBlightBonus( plot )	
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_PIG"):
				self.doBlightBonus( plot )			
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_SHEEP"):
				self.doBlightBonus( plot )	
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_FUR"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_COTTON"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_TOBACCO"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_BANANA"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_DYE"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_IVORY"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_SILK"):
				self.doBlightBonus( plot )
			if plot.getBonusType(-1) == CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), "BONUS_SPICES"):
				self.doBlightBonus( plot )
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_FOREST"):
				self.doBlightFeature( plot )
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_JUNGLE"):
				self.doBlightFeature( plot )
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_OBSIDIAN_PLAINS"):
				self.doBlightFeature( plot )
			if plot.getFeatureType() == CvUtil.findInfoTypeNum(gc.getFeatureInfo, gc.getNumFeatureInfos(), "FEATURE_SCRUB"):
				self.doBlightFeature( plot )

#Melts Tundra to Grassland		
		if plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_TUNDRA"):
			plot.setTerrainType(CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_GRASS"), 1, 1)

#Dries Out Marsh		
		if plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_MARSH"):
			plot.setTerrainType(CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_PLAINS"), 1, 1)

#Melts Snow to Tundra	
		if plot.getTerrainType() == CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_SNOW"):
			plot.setTerrainType(CvUtil.findInfoTypeNum(gc.getTerrainInfo, gc.getNumTerrainInfos(), "TERRAIN_TUNDRA"), 1, 1)

	def doBlightBonus( self, plot ):
		iBonus = plot.getBonusType(-1)
		szTitle = localText.getText("TXT_KEY_GODS_BLIGHT_BONUS", (gc.getBonusInfo(iBonus).getDescription(), ))
		CyInterface().addMessage(plot.getOwner(), False, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_BLIGHT", InterfaceMessageTypes.MESSAGE_TYPE_INFO, gc.getBonusInfo(iBonus).getButton(), gc.getInfoTypeForString("COLOR_GREEN"), plot.getX(), plot.getY(), True, True)	
		plot.setBonusType(-1)

	def doBlightFeature( self, plot ):
		iFeature = plot.getFeatureType()
		szTitle = localText.getText("TXT_KEY_GODS_BLIGHT_FEATURE", (gc.getFeatureInfo(iFeature).getDescription(), ))
		CyInterface().addMessage(plot.getOwner(), False, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_BLIGHT", InterfaceMessageTypes.MESSAGE_TYPE_INFO, gc.getFeatureInfo(iFeature).getButton(), gc.getInfoTypeForString("COLOR_GREEN"), plot.getX(), plot.getY(), True, True)	
		plot.setFeatureType(-1, 0)		

# Lawyer
	def doEventLawyer( self, pCity, pUnit ):
		pPlayer = gc.getPlayer( pCity.getOwner( ) )
		iPlayer = pPlayer.getID( )
		
		#Gets a list of all corporations in the city
		#CyInterface().playGeneralSound("AS2D_BUILD_BANK")
		lCityCorporations = [ ]
		for iCorpLoop in range(gc.getNumCorporationInfos( )):
			if pCity.isHasCorporation( iCorpLoop ):
				iHC = 0
				if pCity.isHeadquartersByType(iCorpLoop):
					iHC = -75

		# Does litigation succeed
		irandom = gc.getGame().getSorenRandNum(100,"")
		if irandom < 95 - ((len( lCityCorporations ))*5) + iHC:

			# Removes buildings
			for iBuildingLoop in range(gc.getNumBuildingInfos( )):
				if pCity.isHasBuilding( iBuildingLoop ):
					pBuilding = gc.getBuildingInfo( iBuildingLoop )
					iRequiredCorporation = pBuilding.getFoundsCorporation( )
					for iCorpLoop in range(gc.getNumCorporationInfos()):
						if iRequiredCorporation == iCorpLoop:
							pCity.setNumRealBuilding ( iBuildingLoop,0 )


			# Loop through all corporations, remove them from the city
			for iCorpLoop in range(gc.getNumCorporationInfos()):
				pCity.setHasCorporation(iCorpLoop, 0, 0, 0)

			# Litigation succeeds
			CyInterface().addMessage(iPlayer,False,25,CyTranslator().getText("TXT_KEY_MESSAGE_LITIGATION",(pCity.getName(),)),"AS2D_BUILD_BANK",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(8),pCity.getX(),pCity.getY(),True,True)

		# Litigation fails
		else:
			CyInterface().addMessage(iPlayer,False,25,CyTranslator().getText("TXT_KEY_MESSAGE_LITIGATION_FAIL",(pCity.getName(),)),"AS2D_SABOTAGE",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(7),pCity.getX(),pCity.getY(),True,True)

		# Unit expended
		pUnit.kill( 0, -1 )


# Bubonic Plague
	def doEventPlague( self, targetPlot ):
		plotX = targetPlot.getX()
		plotY = targetPlot.getY()
		city = gc.getMap( ).plot( plotX, plotY ).getPlotCity()
		szCityName = city.getName()		
		iPlagueEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "EFFECT_GODS_PLAGUE" )
		iCoords = ( plotX, plotY )

		self.lPlagueCities.append( iCoords )
		self.doPlagueEffect( city )
		plot = city.plot()
		szTitle = localText.getText("TXT_KEY_GODS_PLAGUE_START", ( szCityName, ))
		CyInterface().addMessage(plot.getOwner(), True, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_PLAGUE", InterfaceMessageTypes.MESSAGE_TYPE_INFO, gc.getEffectInfo(iPlagueEffect).getButton(), gc.getInfoTypeForString("COLOR_RED"), plot.getX(), plot.getY(), True, True)	
		#plot 5 is Plague
		gc.getMap().plotByIndex(0).setScriptData("Plague On")
		#plot 6 is Black Death Counter
		gc.getMap().plotByIndex(1).setScriptData("10")
		
	def doPlagueSpread(self):
		iPlagueEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "EFFECT_GODS_PLAGUE" )

		#counts down turns remaining of plague
		if gc.getMap().plotByIndex(1).getScriptData() == "10":
			gc.getMap().plotByIndex(1).setScriptData("9")
		elif gc.getMap().plotByIndex(1).getScriptData() == "9":
			gc.getMap().plotByIndex(1).setScriptData("8")			
		elif gc.getMap().plotByIndex(1).getScriptData() == "8":
			gc.getMap().plotByIndex(1).setScriptData("7")
		elif gc.getMap().plotByIndex(1).getScriptData() == "7":
			gc.getMap().plotByIndex(1).setScriptData("6")
		elif gc.getMap().plotByIndex(1).getScriptData() == "6":
			gc.getMap().plotByIndex(1).setScriptData("5")
		elif gc.getMap().plotByIndex(1).getScriptData() == "5":
			gc.getMap().plotByIndex(1).setScriptData("4")
		elif gc.getMap().plotByIndex(1).getScriptData() == "4":
			gc.getMap().plotByIndex(1).setScriptData("3")
		elif gc.getMap().plotByIndex(1).getScriptData() == "3":
			gc.getMap().plotByIndex(1).setScriptData("2")
		elif gc.getMap().plotByIndex(1).getScriptData() == "2":
			gc.getMap().plotByIndex(1).setScriptData("1")
		else:
			gc.getMap().plotByIndex(0).setScriptData("")
		
		lPlayers = PyGame().getCivPlayerList()
		lValidCities = []
		
			
		for iCoords in self.lPlagueCities:
			city = gc.getMap( ).plot( iCoords[0], iCoords[1] ).getPlotCity( )
			iTradeRoutes = city.getTradeRoutes()
			for i in range(iTradeRoutes):
				tradeCity = city.getTradeCity(i)
				distance = self.doGetDistanceBetweenPlots( city.getX(), city.getY(), tradeCity.getX(), tradeCity.getY() )
				iTradeCityCoords = ( tradeCity.getX( ), tradeCity.getY( ) )
				if self.lPlagueCities.count( iTradeCityCoords ) == 0:
#				if tradeCity.getScriptData() != "Plague City":
					if distance < 10:
						if self.getRandomNumber(5) == 0:
							self.lPlagueCities.append( iTradeCityCoords )
#							tradeCity.setScriptData("Plague City")
							self.doPlagueEffect(tradeCity)
							plot = tradeCity.plot()
							szCityName = tradeCity.getName()
							szTitle = localText.getText("TXT_KEY_GODS_PLAGUE_SPREAD", ( szCityName, ))
							CyInterface().addMessage(plot.getOwner(), True, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_PLAGUE", InterfaceMessageTypes.MESSAGE_TYPE_INFO, gc.getEffectInfo(iPlagueEffect).getButton(), gc.getInfoTypeForString("COLOR_RED"), plot.getX(), plot.getY(), True, True)	

	def doPlagueDecay(self):
		lPlayers = PyGame().getCivPlayerList()
		lValidCities = []
		iPlagueEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "EFFECT_GODS_PLAGUE" )

		
#		for player in lPlayers:
#			lCities = player.getCityList()
#			for city in lCities:
#				if city.getScriptData() == "Plague City":
#					lValidCities.append(city)
		
		for iCoords in self.lPlagueCities:
			city = gc.getMap( ).plot( iCoords[0], iCoords[1] ).getPlotCity( )
			if self.getRandomNumber(1) == 0:
				self.doPlagueRemoveEffect( city )
				self.lPlagueCities.remove( iCoords )
#				city.setScriptData("")
				szCityName = city.getName()
				plot = city.plot()
				szTitle = localText.getText("TXT_KEY_GODS_PLAGUE_DECAY", ( szCityName, ))
				CyInterface().addMessage(plot.getOwner(), True, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_DISCOVERBONUS", InterfaceMessageTypes.MESSAGE_TYPE_INFO, gc.getEffectInfo(iPlagueEffect).getButton(), gc.getInfoTypeForString("COLOR_RED"), plot.getX(), plot.getY(), True, True)	


	def doPlagueEffect(self, city):
		gc.getPlayer(city.getOwner()).getCity(city.getID()).changeExtraHealth(-8)

	def doPlagueRemoveEffect(self, city):
		gc.getPlayer(city.getOwner()).getCity(city.getID()).changeExtraHealth(8)
		
	def getRandomNumber(self, int):
		return CyGame().getSorenRandNum(int, "Gods")

	def doGetDistanceBetweenPlots ( self, iX1, iY1, iX2, iY2):
		return abs(iX1 - iX2) + abs(iY1 - iY2)

# Doctor and Statesman AI
	def doAIOperations( self, iPlayer ):
		lPlayers = PyGame().getCivPlayerList( )
		player = gc.getPlayer( iPlayer )
		lUnits = PyPlayer( iPlayer ).getUnitList( )
		playerStateReligion = gc.getPlayer( iPlayer ).getStateReligion( )
		
		for unit in lUnits:
			if unit.getUnitType( ) == CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_GREAT_DOCTOR"):
				if ( playerStateReligion == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ) or ( playerStateReligion != CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ):
					for iOpponent in lPlayers:
						if iOpponent.getID() != iPlayer:
							iAttitude = player.AI_getAttitude( iOpponent.getID() )
							if iAttitude == AttitudeTypes.ATTITUDE_ANNOYED:
								if self.getRandomNumber( 15 ) == 0:
									self.doAITargetOpponentPlague( iOpponent.getID(), iPlayer )
									unit.kill( 0, unit.getOwner())
							if iAttitude == AttitudeTypes.ATTITUDE_FURIOUS:
								if self.getRandomNumber( 5 ) == 0:
									unit.kill( 0, unit.getOwner())
									self.doAITargetOpponentPlague( iOpponent.getID(), iPlayer )


		for unit in lUnits:
			if unit.getUnitType( ) == CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_ANDROMEDA"):
				if ( playerStateReligion == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ) or ( playerStateReligion != CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ):
					for iOpponent in lPlayers:
						if iOpponent.getID() != iPlayer:
							iAttitude = player.AI_getAttitude( iOpponent.getID() )
							if iAttitude == AttitudeTypes.ATTITUDE_ANNOYED:
								if self.getRandomNumber( 15 ) == 0:
									self.doAITargetOpponentBlight( iOpponent.getID(), iPlayer )	
									unit.kill( 0, unit.getOwner())
							if iAttitude == AttitudeTypes.ATTITUDE_FURIOUS:
								if self.getRandomNumber( 5 ) == 0:
									unit.kill( 0, unit.getOwner())
									self.doAITargetOpponentBlight( iOpponent.getID(), iPlayer )

		for unit in lUnits:
			if unit.getUnitType( ) == CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_STATESMAN"):
				if ( playerStateReligion == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ) or ( playerStateReligion != CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ):
					for iOpponent in lPlayers:
						if iOpponent.getID() != iPlayer:
							iAttitude = player.AI_getAttitude( iOpponent.getID() )
							if iAttitude == AttitudeTypes.ATTITUDE_FRIENDLY:
								if self.getRandomNumber( 15 ) == 0:
									self.doAITargetOpponentMeteor( iOpponent.getID(), iPlayer )
									unit.kill( 0, unit.getOwner())
							if iAttitude == AttitudeTypes.ATTITUDE_PLEASED:
								if self.getRandomNumber( 5 ) == 0:
									unit.kill( 0, unit.getOwner())
									self.doAITargetOpponentMeteor( iOpponent.getID(), iPlayer )
		
	def doAITargetOpponentEarthquake( self, iOpponent, iPlayer ):
		lCities = PyPlayer( iOpponent ).getCityList()
		numCities = len( lCities )
		pCity = lCities[ self.getRandomNumber( numCities ) ]
		
		if not self.doAnReligionCheck( iOpponent, iPlayer ):
			self.doEventEarthquake( pCity.plot() )
		
	def doAITargetOpponentPlague( self, iOpponent, iPlayer ):
		lCities = PyPlayer( iOpponent ).getCityList()
		numCities = len( lCities )
		pCity = lCities[ self.getRandomNumber( numCities ) ]
		if not self.doAnReligionCheck( iOpponent, iPlayer ):
			self.doEventPlague( pCity.plot() )
	
	def doAITargetOpponentMeteor( self, iOpponent, iPlayer ):
		lCities = PyPlayer( iOpponent ).getCityList()
		numCities = len( lCities )
		pCity = lCities[ self.getRandomNumber( numCities ) ]

		if not self.doAnReligionCheck( iOpponent, iPlayer ):
			self.doEventMinorMeteorStrike( pCity.plot() )
		
	def doAITargetOpponentBlight( self, iOpponent, iPlayer ):
		lCities = PyPlayer( iOpponent ).getCityList()
		numCities = len( lCities )
		pCity = lCities[ self.getRandomNumber( numCities ) ]

		if not self.doAnReligionCheck( iOpponent, iPlayer ):
			self.doEventBlight( pCity.plot() )
		
	def doAITargetOpponentTsunami( self, iOpponent, unit, iPlayer ):
		lAllCities = PyPlayer( iOpponent ).getCityList()
		lValidCities = [ ]
		
		for pCity in lAllCities:
			if pCity.plot().isCoastalLand():
				lValidCities.append( pCity )
		if len( lValidCities ) > 0:
			numCities = len( lValidCities )
			pChosenCity = lValidCities[ self.getRandomNumber( numCities ) ]
			if not self.doAnReligionCheck( iOpponent, iPlayer ):
				self.doEventTsunami( pChosenCity.plot() )
			unit.kill( 0, unit.getOwner( ) )

## Bath Check?

	def doAnReligionCheck( self, iOpponent, iPlayer ):
		pyTargetPlayer = PyPlayer( iOpponent )
		pDefendingPlayer = gc.getPlayer( iOpponent )
		pAttackingPlayer = gc.getPlayer( iPlayer )
		
		if ( pyTargetPlayer.getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ) or ( pyTargetPlayer.getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ):

#			b_bath = gc.getInfoTypeForString("BUILDING_GREAT_BATH")
#			obsoleteTech = gc.getBuildingInfo(b_bath).getObsoleteTech()
#
#			if ( gc.getTeam(pyTargetPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
#				for iCity in range(pyTargetPlayer.getNumCities()):
#					ppCity = pyTargetPlayer.getCity(iCity)
#					if ppCity.getNumActiveBuilding(b_bath) == true:
#						iBathEffect = CvUtil.findInfoTypeNum( gc.getEffectInfo, gc.getNumEffectInfos( ), "EFFECT_SUBMARINE_HIT_SPLASH" )
#						CyEngine().triggerEffect( iBathEffect, plotPoint )
#
			lUnits = pyTargetPlayer.getUnitList()
			for unit in lUnits:
				if unit.getUnitType() == CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_KRAKEN"):
					unit.kill( 0, unit.getOwner( ) )


					if not ( pDefendingPlayer.isHuman( ) ):
						pDefendingPlayer.AI_changeAttitudeExtra( iPlayer, -1 )
						szAttackedCiv = pDefendingPlayer.getCivilizationDescriptionKey()
						szAttackingCiv = pAttackingPlayer.getCivilizationDescriptionKey()
						szTitle = localText.getText("TXT_KEY_GODS_DISASTER_ANGER_TWO1", (szAttackedCiv , szAttackingCiv, ))
						CyInterface().addImmediateMessage( szTitle , None)			
					else:
						szAttackedCiv = pDefendingPlayer.getCivilizationDescriptionKey()
						szAttackingCiv = pAttackingPlayer.getCivilizationDescriptionKey()
						szTitle = localText.getText("TXT_KEY_GODS_DISASTER_HUMAN_TWO1", (szAttackedCiv , szAttackingCiv, ))
						CyInterface().addImmediateMessage( szTitle , None)	
					return 1

		return 0	
	

	def clearGodsVariables( self ):
		
		self.iStarAlignment = 0
		self.lPlagueCities = [ ]
		self.lTsunamiCoor = [ ]


	def setGodsVariables(self):
		pPlayer = gc.getPlayer(0)
		
		szScriptData = []

		szScriptData.append(self.iStarAlignment)
		szScriptData.append( self.lPlagueCities )	
		
		print szScriptData

		# Save Script Data - Score
		pPlayer.setScriptData(pickle.dumps(szScriptData))	
		
	def getGodsVariables(self):
		
		pPlayer = gc.getPlayer(0)
		lPlagueCityCoord = [ ]
		
		# Load Script Data - Score
		szScriptData = pickle.loads(pPlayer.getScriptData())
		
		self.iStarAlignment = szScriptData[0]
		self.lPlagueCities = szScriptData[1]
		
		return 

	
	def doInquisitorPersecution( self, pCity, pUnit ):
		pPlayer = gc.getPlayer( pCity.getOwner( ) )
		iPlayer = pPlayer.getID( )

		
		# gets a list of all religions in the city except state religion
		#CyInterface().playGeneralSound("AS3D_UN_CHRIST_MISSIONARY_ACTIVATE")
		lCityReligions = [ ]
		for iReligionLoop in range(gc.getNumReligionInfos( )):
			if pCity.isHasReligion( iReligionLoop ):
				if pPlayer.getStateReligion( ) != iReligionLoop:
					lCityReligions.append( iReligionLoop )
					iHC = 0
					if pCity.isHolyCityByType(iReligionLoop):
						iHC = -75
				else:
					if pCity.isHolyCityByType(iReligionLoop):
						iHC = 15					

		# Does Persecution succeed
		irandom = gc.getGame().getSorenRandNum(100,"")
		if irandom < 95 - ((len( lCityReligions ))*5) + iHC:
			
			# increases Anger for all AIs in City Religion List
			if len( lCityReligions ) > 0:
				lPlayers =PyGame().getCivPlayerList()
				for iAI_PlayersLoop in range( len(lPlayers) ):
					pSecondPlayer = lPlayers[iAI_PlayersLoop]
					iSecondPlayer = pSecondPlayer.getID( )
					for iAIAngerLoop in range( len( lCityReligions ) ):
						pReligion = gc.getReligionInfo( lCityReligions[iAIAngerLoop] )
						if pSecondPlayer.getStateReligion( ) == lCityReligions[iAIAngerLoop]:
							pSecondPlayer.AI_changeAttitude( pPlayer.getID( ), -1 )
							CyInterface().addMessage(iSecondPlayer,False,25,CyTranslator().getText("TXT_KEY_MESSAGE_INQUISITION_GLOBAL",(pCity.getName(),pReligion.getText())),"AS2D_PLAGUE",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
						else:
							if iSecondPlayer != iPlayer:
								lCities = PyPlayer( iSecondPlayer ).getCityList( )
								for iCityloop in range( pSecondPlayer.getNumCities() ):
									pPlayerCity = lCities[ iCityloop ]
									if pPlayerCity.isHolyCityByType(lCityReligions[iAIAngerLoop]):
										CyInterface().addMessage(iSecondPlayer,False,25,CyTranslator().getText("TXT_KEY_MESSAGE_INQUISITION_GLOBAL",(pCity.getName(),pReligion.getText())),"AS2D_PLAGUE",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(7),pCity.getX(),pCity.getY(),True,True)

			# removes buildings
			for iBuildingLoop in range(gc.getNumBuildingInfos( )):
				if pCity.isHasBuilding( iBuildingLoop ):
					pBuilding = gc.getBuildingInfo( iBuildingLoop )
					iRequiredReligion = pBuilding.getPrereqReligion( )
					for iReligionLoop in range(gc.getNumReligionInfos()):
						if iReligionLoop != pPlayer.getStateReligion( ):
							if iRequiredReligion == iReligionLoop:
								pCity.setNumRealBuilding ( iBuildingLoop,0 )

			# Loop through all religions, remove them from the city
			for iReligionLoop in range(gc.getNumReligionInfos()):
				if iReligionLoop != pPlayer.getStateReligion( ):
					if pCity.isHolyCityByType( iReligionLoop ):
						gc.getGame( ).clearHolyCity( iReligionLoop )
				pCity.setHasReligion(iReligionLoop, 0, 0, 0)
			
			# Add player's state religion
			if ( gc.getPlayer( pUnit.getOwner( ) ).getStateReligion( ) != -1 ):
				iStateReligion = gc.getPlayer( pUnit.getOwner( ) ).getStateReligion( )
				pCity.setHasReligion( iStateReligion, 1, 0, 0 )

			# Persecution succeeds
			CyInterface().addMessage(iPlayer,False,25,CyTranslator().getText("TXT_KEY_MESSAGE_INQUISITION",(pCity.getName(),)),"AS2D_PLAGUE",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(8),pCity.getX(),pCity.getY(),True,True)

		# Persecution fails
		else:
			CyInterface().addMessage(iPlayer,False,25,CyTranslator().getText("TXT_KEY_MESSAGE_INQUISITION_FAIL",(pCity.getName(),)),"AS2D_SABOTAGE",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(7),pCity.getX(),pCity.getY(),True,True)

		# Unit expended
		pUnit.kill( 0, -1 )	

## Copernicus Start Sign Order ##

	def doChooseNewStarAlignment( self ):
		
		self.iStarAlignment = self.getRandomNumber( 11 )
		if self.iStarAlignment == 0:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_RAM", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarWarlordBonus( "NONE" )
		if self.iStarAlignment == 1:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_BULL", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarProductionBonus( "NONE" )
		if self.iStarAlignment == 2:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_TWIN", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarCultureBonus( "NONE" )
		if self.iStarAlignment == 3:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_CRAB", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
		if self.iStarAlignment == 4:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_LION", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarGoldBonus( "NONE" )
		if self.iStarAlignment == 5:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_VIRGIN", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarReligionBonus( "NONE" )
		if self.iStarAlignment == 6:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_SCALE", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarEspionageBonus( "NONE" )
		if self.iStarAlignment == 7:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_SCORPION", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarNegativeBonus( "NONE" )
		if self.iStarAlignment == 8:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_ARCHER", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarDefenseBonus( "NONE" )
		if self.iStarAlignment == 9:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_GOAT", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarFoodBonus( "NONE" )
		if self.iStarAlignment == 10:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_WATER", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarGreatPeopleBonus( "NONE" )
		if self.iStarAlignment == 11:
			szTitle = localText.getText( "TXT_KEY_GODS_STARS_FISH", ( ) )
			CyInterface().addImmediateMessage( szTitle , None)
			self.doStarPopulationBonus( "NONE" )

## Copernicus Star Sign Order ##
## Copernicus Star Bonus ##	
		
	def doStarFoodBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							lCities = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getCityList( )
							for iCity in range( len( lCities ) ):
								pCity = lCities[ iCity ]
								pCity.changeFood( +6 )

	def doStarGoldBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iGold = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getGold( )
							if PyPlayer( lPlayers[ iPlayer ].getID( ) ).getGold( ) > 100:
								PyPlayer( lPlayers[ iPlayer ].getID( ) ).changeGold( iGold//10 )
							else:
								PyPlayer( lPlayers[ iPlayer ].getID( ) ).changeGold( 10 )

	def doStarCultureBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							lCities = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getCityList( )
							for iCity in range( len( lCities ) ):
								pCity = lCities[ iCity ]
								pCity.changeCulture( +10 )

	def doStarProductionBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							lCities = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getCityList( )
							for iCity in range( len( lCities ) ):
								pCity = lCities[ iCity ]
								pCity.changeProduction( +6 )

	def doStarReligionBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iX = ppCity.getX()
							iY = ppCity.getY()

							iMissionary0 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_INQUISITOR')
							iMissionary1 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_JEWISH_MISSIONARY')
							iMissionary2 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_CHRISTIAN_MISSIONARY')
							iMissionary3 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_HINDU_MISSIONARY')
							iMissionary4 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_BUDDHIST_MISSIONARY')
							iMissionary5 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_CONFUCIAN_MISSIONARY')
							iMissionary6 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_TAOIST_MISSIONARY')
							iMissionary7 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_PAGAN_MISSIONARY')
							iMissionary8 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ISLAMIC_MISSIONARY')
							iMissionary9 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ZORO_MISSIONARY')
							iMissionary10 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_SHINTO_MISSIONARY')
							iMissionary11 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ASEN_MISSIONARY')
							iMissionary12 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_RA_MISSIONARY')
							iMissionary13 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_TOLTEC_MISSIONARY')
							iMissionary14 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_VOODOO_MISSIONARY')
							iMissionary15 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_TENGRI_MISSIONARY')
							iMissionary16 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_LEVANT_MISSIONARY')

							for i in range(1):
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_JUDAISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary1, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_CHRISTIANITY") ):
									pNewUnit = pPlayer.initUnit( iMissionary2, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_HINDUISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary3, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_BUDDHISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary4, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_CONFUCIANISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary5, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_TAOISM") ):
									pNewUnit = pPlayer.initUnit( iMissionary6, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_PAGAN") ):
									pNewUnit = pPlayer.initUnit( iMissionary7, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_ISLAM") ):
									pNewUnit = pPlayer.initUnit( iMissionary8, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_ZOR") ):
									pNewUnit = pPlayer.initUnit( iMissionary9, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_SHINTO") ):
									pNewUnit = pPlayer.initUnit( iMissionary10, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_ASEN") ):
									pNewUnit = pPlayer.initUnit( iMissionary11, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_RA") ):
									pNewUnit = pPlayer.initUnit( iMissionary12, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_TOLTEC") ):
									pNewUnit = pPlayer.initUnit( iMissionary13, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_VOODOO") ):
									pNewUnit = pPlayer.initUnit( iMissionary14, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_TENGRI") ):
									pNewUnit = pPlayer.initUnit( iMissionary15, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "RELIGION_LEVANT") ):
									pNewUnit = pPlayer.initUnit( iMissionary16, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )
								if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo, gc.getNumReligionInfos(), "NONE") ):
									pNewUnit = pPlayer.initUnit( iMissionary0, iX, iY, UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.NO_DIRECTION )


	def doStarWarlordBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iImp = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_IMPERIALIST')
							iArc = CvUtil.findInfoTypeNum(gc.getBuildingInfo,gc.getNumBuildingInfos(),'BUILDING_ARC')
							iLegalCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_LEGAL')
							iMartial = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_MART')
							iLegCivic = pPlayer.getCivics(iLegalCivicOption)
							
							if (iLegCivic == iMartial):
								if (pPlayer.hasTrait(iImp)):
									if ppCity.getNumActiveBuilding(iArc) == true:
										pPlayer.changeCombatExperience( +16 )
									elif ppCity.getNumActiveBuilding(iArc) == false:
										pPlayer.changeCombatExperience( +12 )
								else:
									if ppCity.getNumActiveBuilding(iArc) == true:
										pPlayer.changeCombatExperience( +12 )
									elif ppCity.getNumActiveBuilding(iArc) == false:
										pPlayer.changeCombatExperience( +8 )
							elif (iLegCivic != iMartial):
								if (pPlayer.hasTrait(iImp)):
									if ppCity.getNumActiveBuilding(iArc) == true:
										pPlayer.changeCombatExperience( +12 )
									elif ppCity.getNumActiveBuilding(iArc) == false:
										pPlayer.changeCombatExperience( +8 )
								else:
									if ppCity.getNumActiveBuilding(iArc) == true:
										pPlayer.changeCombatExperience( +8 )
									elif ppCity.getNumActiveBuilding(iArc) == false:
										pPlayer.changeCombatExperience( +4 )
							else:
								pPlayer.changeCombatExperience( +4 )

	def doStarDefenseBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iX = ppCity.getX()
							iY = ppCity.getY()

							iDefense0 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_WARRIOR')
							iDefense1 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ARCHER')
							iDefense2 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_LONGBOWMAN')
							iDefense3 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MUSKETMAN')
							iDefense4 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_RIFLEMAN')
							iDefense5 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MACHINE_GUN')
							iDefense6 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MARINE')
							iDefense7 = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MECHANIZED_INFANTRY')

							iDefense0a = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_INCAN_QUECHUA')

							iDefense1a = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MALI_SKIRMISHER')
							iDefense1b = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_BABYLON_BOWMAN')
							iDefense1c = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_NUBIAN_MEDJAY')
							iDefense1d = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_AKKADIAN')

							iDefense2a = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_KONGO')

							iDefense3a = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ETHIOPIAN_OROMO_WARRIOR')
							iDefense3b = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_OTTOMAN_JANISSARY')
							iDefense3c = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_FRENCH_MUSKETEER')

							iDefense4a = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ENGLISH_REDCOAT')
							iDefense4b = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_AUSTRIA_KUKINFANTRIE')
							iDefense4c = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_SWEDEN')

							iDefense6a = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_AMERICAN_NAVY_SEAL')

							iTech1 = gc.getInfoTypeForString("TECH_ROBOTICS")
							iTech2 = gc.getInfoTypeForString("TECH_INDUSTRIALISM")
							iTech3 = gc.getInfoTypeForString("TECH_RAILROAD")
							iTech4 = gc.getInfoTypeForString("TECH_RIFLING")
							iTech5 = gc.getInfoTypeForString("TECH_GUNPOWDER")
							iTech6 = gc.getInfoTypeForString("TECH_FEUDALISM")
							iTech7 = gc.getInfoTypeForString("TECH_ARCHERY")

							for i in range(1):
								if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech1) == true or obsoleteTech == 1 ):
									pNewUnit = pPlayer.initUnit( iDefense7, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
								elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech1) == false or obsoleteTech == -1 ):

									if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech2) == true or obsoleteTech == 1 ):
										if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_AMERICA")):
											pNewUnit = pPlayer.initUnit( iDefense6a, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
										elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_AMERICA")):
											pNewUnit = pPlayer.initUnit( iDefense6, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
									elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech2) == false or obsoleteTech == -1 ):

										if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech3) == true or obsoleteTech == 1 ):
											pNewUnit = pPlayer.initUnit( iDefense5, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
										elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech3) == false or obsoleteTech == -1 ):

											if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech4) == true or obsoleteTech == 1 ):
												if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_ENGLAND")):
													pNewUnit = pPlayer.initUnit( iDefense4a, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
												elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_AUSTRIA")):
													pNewUnit = pPlayer.initUnit( iDefense4b, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
												elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_NEW_FRANCE")):
													pNewUnit = pPlayer.initUnit( iDefense4c, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
												elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_ENGLAND")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_AUSTRIA")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_NEW_FRANCE")):
													pNewUnit = pPlayer.initUnit( iDefense4, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
											elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech4) == false or obsoleteTech == -1 ):

												if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech5) == true or obsoleteTech == 1 ):
													if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_ETHIOPIA")):
														pNewUnit = pPlayer.initUnit( iDefense3a, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
													elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_OTTOMAN")):
														pNewUnit = pPlayer.initUnit( iDefense3b, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
													elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_FRANCE")):
														pNewUnit = pPlayer.initUnit( iDefense3c, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
													elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_ETHIOPIA")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_OTTOMAN")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_FRANCE")):
														pNewUnit = pPlayer.initUnit( iDefense3, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
												elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech5) == false or obsoleteTech == -1 ):

													if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech6) == true or obsoleteTech == 1 ):
														if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_KONGO")):
															pNewUnit = pPlayer.initUnit( iDefense2a, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
														elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_KONGO")):
															pNewUnit = pPlayer.initUnit( iDefense2, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
													elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech6) == false or obsoleteTech == -1 ):

														if ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech7) == true or obsoleteTech == 1 ):
															if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_MALI")):
																pNewUnit = pPlayer.initUnit( iDefense1a, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
															elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_BABYLON")):
																pNewUnit = pPlayer.initUnit( iDefense1b, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
															elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_NUBIA")):
																pNewUnit = pPlayer.initUnit( iDefense1c, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
															elif (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_AKKAD")):
																pNewUnit = pPlayer.initUnit( iDefense1d, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
															elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_MALI")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_BABYLON")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_NUBIA")) and (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_AKKAD")):
																pNewUnit = pPlayer.initUnit( iDefense1, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
														elif ( gc.getTeam(pPlayer.getTeam()).isHasTech(iTech7) == false or obsoleteTech == -1 ):

															if (pPlayer.getCivilizationType() == gc.getInfoTypeForString("CIVILIZATION_INCA")):
																pNewUnit = pPlayer.initUnit( iDefense0a, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )
															elif (pPlayer.getCivilizationType() != gc.getInfoTypeForString("CIVILIZATION_INCA")):
																pNewUnit = pPlayer.initUnit( iDefense0, iX, iY, UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.NO_DIRECTION )


	def doStarEspionageBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iX = ppCity.getX()
							iY = ppCity.getY()

							iSpy = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_SPY')

							for i in range(1):
								pNewUnit = pPlayer.initUnit( iSpy, iX, iY, UnitAITypes.UNITAI_SPY, DirectionTypes.NO_DIRECTION )

	def doStarGreatPeopleBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							iRelCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_RELIGION')
							iPacifist = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_PACIFISM')
							iRelCivic = pPlayer.getCivics(iRelCivicOption)
							iPhi = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_PHILOSOPHICAL')
							iNational = CvUtil.findInfoTypeNum(gc.getBuildingInfo,gc.getNumBuildingInfos(),'BUILDING_NATIONAL_EPIC')

							if (iRelCivic == iPacifist):
								if (pPlayer.hasTrait(iPhi)):
									if ppCity.getNumActiveBuilding(iNational) == true:
										ppCity.changeGreatPeopleProgress( +12 )
									elif ppCity.getNumActiveBuilding(iNational) == false:
										ppCity.changeGreatPeopleProgress( +9 )
								else:
									if ppCity.getNumActiveBuilding(iNational) == true:
										ppCity.changeGreatPeopleProgress( +9 )
									elif ppCity.getNumActiveBuilding(iNational) == false:
										ppCity.changeGreatPeopleProgress( +6 )
							elif (iRelCivic != iPacifist):
								if (pPlayer.hasTrait(iPhi)):
									if ppCity.getNumActiveBuilding(iNational) == true:
										ppCity.changeGreatPeopleProgress( +9 )
									elif ppCity.getNumActiveBuilding(iNational) == false:
										ppCity.changeGreatPeopleProgress( +6 )
								else:
									if ppCity.getNumActiveBuilding(iNational) == true:
										ppCity.changeGreatPeopleProgress( +6 )
									elif ppCity.getNumActiveBuilding(iNational) == false:
										ppCity.changeGreatPeopleProgress( +3 )
							else:
								ppCity.changeGreatPeopleProgress( +3 )

	def doStarPopulationBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							ppCity.changePopulation( +1 )

	def doStarNegativeBonus( self, szReligion ):
		lPlayers = PyGame().getCivPlayerList( )
		
		for iPlayer in range( len( lPlayers ) ):
			if ( lPlayers[ iPlayer ].getStateReligion( ) == CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ) or ( lPlayers[ iPlayer ].getStateReligion( ) != CvUtil.findInfoTypeNum(gc.getReligionInfo,gc.getNumReligionInfos(), szReligion ) ):

				pPlayer = gc.getPlayer(iPlayer)
				b_coperniucus = gc.getInfoTypeForString("BUILDING_COPERNICUS")
				obsoleteTech = gc.getBuildingInfo(b_coperniucus).getObsoleteTech()

				if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
					for iCity in range(pPlayer.getNumCities()):
						ppCity = pPlayer.getCity(iCity)
						if ppCity.getNumActiveBuilding(b_coperniucus) == true:

							lCities = PyPlayer( lPlayers[ iPlayer ].getID( ) ).getCityList( )
							for iCity in range( len( lCities ) ):
								pCity = lCities[ iCity ]
								pCity.changeCulture( -5 )
								pCity.changeProduction( -2 )
								pCity.changeFood( -2 )

## Copernicus Star Bonus ##

####### Next War Functions ######

	def doCheckDepletion(self):
		self.doCheckWorkedResource("BONUS_ALUMINUM", "IMPROVEMENT_MINE", 880)
		self.doCheckWorkedResource("BONUS_BANANA", "IMPROVEMENT_PLANTATION", 1120)
		self.doCheckWorkedResource("BONUS_CLAM", "IMPROVEMENT_FISHING_BOATS", 880)
		self.doCheckWorkedResource("BONUS_COAL", "IMPROVEMENT_MINE", 880)
		self.doCheckWorkedResource("BONUS_COPPER", "IMPROVEMENT_MINE", 880)
		self.doCheckWorkedResource("BONUS_CORN", "IMPROVEMENT_FARM", 1120)
		self.doCheckWorkedResource("BONUS_COW", "IMPROVEMENT_PASTURE", 1120)
		self.doCheckWorkedResource("BONUS_CRAB", "IMPROVEMENT_FISHING_BOATS", 1120)
		self.doCheckWorkedResource("BONUS_DEER", "IMPROVEMENT_CAMP", 780)
		self.doCheckWorkedResource("BONUS_DYE", "IMPROVEMENT_PLANTATION", 1120)
		self.doCheckWorkedResource("BONUS_FISH", "IMPROVEMENT_FISHING_BOATS", 880)
		self.doCheckWorkedResource("BONUS_FUR", "IMPROVEMENT_CAMP", 560)
		self.doCheckWorkedResource("BONUS_GEMS", "IMPROVEMENT_MINE", 1120)
		self.doCheckWorkedResource("BONUS_GOLD", "IMPROVEMENT_MINE", 880)
		self.doCheckWorkedResource("BONUS_HORSE", "IMPROVEMENT_PASTURE", 1120)
		self.doCheckWorkedResource("BONUS_IRON", "IMPROVEMENT_MINE", 880)
		self.doCheckWorkedResource("BONUS_IVORY", "IMPROVEMENT_CAMP", 560)
		self.doCheckWorkedResource("BONUS_MARBLE", "IMPROVEMENT_QUARRY", 880)
		self.doCheckWorkedResource("BONUS_OIL", "IMPROVEMENT_WELL", 880)
		self.doCheckWorkedResource("BONUS_OIL", "IMPROVEMENT_OFFSHORE_PLATFORM", 880)
		self.doCheckWorkedResource("BONUS_PIG", "IMPROVEMENT_PASTURE", 1120)
		self.doCheckWorkedResource("BONUS_RICE", "IMPROVEMENT_FARM", 1120)
		self.doCheckWorkedResource("BONUS_SILK", "IMPROVEMENT_PLANTATION", 1120)
		self.doCheckWorkedResource("BONUS_SILVER", "IMPROVEMENT_MINE", 880)
		self.doCheckWorkedResource("BONUS_SPICES", "IMPROVEMENT_PLANTATION", 1120)
		self.doCheckWorkedResource("BONUS_STONE", "IMPROVEMENT_QUARRY", 880)
		self.doCheckWorkedResource("BONUS_SUGAR", "IMPROVEMENT_PLANTATION", 1120)
		self.doCheckWorkedResource("BONUS_URANIUM", "IMPROVEMENT_MINE", 880)
		self.doCheckWorkedResource("BONUS_WHALE", "IMPROVEMENT_WHALING_BOATS", 560)
		self.doCheckWorkedResource("BONUS_WHEAT", "IMPROVEMENT_FARM", 1120)
		self.doCheckWorkedResource("BONUS_WINE", "IMPROVEMENT_WINERY", 1120)
		
		
	def doCheckWorkedResource(self, bonus, improvement, randomInt):
			
		iBonus = CvUtil.findInfoTypeNum(gc.getBonusInfo, gc.getNumBonusInfos(), bonus)
		
		if (iBonus == -1):
			return
		
		lValidPlots = self.getPlotListbyBonus(iBonus)
		
		if len(lValidPlots) == 0:
			return

		for plot in lValidPlots:
			if plot.getImprovementType() == CvUtil.findInfoTypeNum(gc.getImprovementInfo, gc.getNumImprovementInfos(), improvement):
				if self.getRandomNumber(randomInt) == 0:
					
					pBonusInfo = gc.getBonusInfo(iBonus)
					
					plot.setBonusType(-1)
					
					szTitle = localText.getText("TEXT_KEY_NEXT_WAR_RESOURCE_DEPLETED_TITLE", ())
					#szText = localText.getText("TEXT_KEY_NEXT_WAR_RESOURCE_DEPLETED", (pBonusInfo.getDescription(), plot.getX(), plot.getY()))
										
					CyInterface().addMessage(plot.getOwner(), False, gc.getEVENT_MESSAGE_TIME(), szTitle, "AS2D_DISCOVERBONUS", InterfaceMessageTypes.MESSAGE_TYPE_MINOR_EVENT, pBonusInfo.getButton(), gc.getInfoTypeForString("COLOR_GREEN"), plot.getX(), plot.getY(), True, True)	
					
					

	def getPlotListbyBonus(self, iBonus):
		lPlots = []
		totalPlots = gc.getMap().numPlots()
		
		for i in range(totalPlots):
			plot = gc.getMap().plotByIndex(i)
			iOwner = plot.getOwner()
			if (iOwner != -1):
				if plot.getBonusType(gc.getPlayer(iOwner).getTeam()) == iBonus:
					lPlots.append(plot)
		return lPlots

	def getRandomNumber(self, int):
		return CyGame().getSorenRandNum(int, "Next War")
		
			
#################### Popups ####################
#		
#	def addPopup(self, szTitle, szText):
#		
#		# Don't display popups for autoplay games
#		if (gc.getPlayer(CyGame().getActivePlayer()).isAlive()):
#			
#			popup = PyPopup.PyPopup(-1)
#			popup.setHeaderString(szTitle)
#			popup.setBodyString(szText)
#			popup.launch(true, PopupStates.POPUPSTATE_QUEUED)
#
#	def initScriptData(self):
#		
#		# Set default script data manually since we need defaults for all values in the array before any functions can be called on them
#		iDefaultNumNukesFired = 0
#		
#		aScriptData = [iDefaultNumNukesFired]
#		CyGame().setScriptData(pickle.dumps(aScriptData))
#		
#	def getGameNumNukes(self):
#		aszScriptData = pickle.loads(CyGame().getScriptData())
#		iNumNukes = aszScriptData[0]		 # Num Nukes Fired is 0th element
#		return iNumNukes
#	def setGameNumNukes(self, iValue):
#		aszScriptData = pickle.loads(CyGame().getScriptData())
#		aszScriptData[0] = iValue
#		CyGame().setScriptData(pickle.dumps(aszScriptData))
#		
#		self.checkNukeStuff(aszScriptData[0])
#		
#	def changeGameNumNukes(self, iChange):
#		aszScriptData = pickle.loads(CyGame().getScriptData())
#		aszScriptData[0] = aszScriptData[0] + iChange
#		CyGame().setScriptData(pickle.dumps(aszScriptData))
#		
#		self.checkNukeStuff(aszScriptData[0])
#		
#	def checkNukeStuff(self, iNumNukes):
#		
#		print("iNumNukes")
#		print(iNumNukes)
#		print("g_iNumNukesGameOver")
#		print(g_iNumNukesGameOver)
#		
#		# Amount of nukes matches Warning message
#		if (iNumNukes == g_iNumNukesWarningMessage):
#			
#			self.addPopup("", localText.getText("TXT_KEY_ROUND_THREE_NUKE_WARNING", ()))
#		
#		# Amount of nukes matches Game over
#		elif (iNumNukes == g_iNumNukesGameOver):
#			
#			# Show the screen that shows the world 'sploding
#			CvScreensInterface.showNextWarScreen()
#			
#			CyGame().setWinner(18, 3)
#
#	def addPopup(self, szTitle, szText):
#		
#		# Don't display popups for autoplay games
#		if (gc.getPlayer(CyGame().getActivePlayer()).isAlive()):
#			
#			popup = PyPopup.PyPopup(-1)
#			popup.setHeaderString(localText.getText(szTitle, ()))
#			popup.setBodyString(localText.getText(szText, ()))
#			popup.launch(true, PopupStates.POPUPSTATE_QUEUED)
#

	def addPopup(self, szTitle, szText):
		
		# Don't display popups for autoplay games
		if (gc.getPlayer(gc.getGame().getActivePlayer()).isAlive()):
			
			popup = PyPopup.PyPopup(-1)
			popup.setHeaderString(szTitle)
			popup.setBodyString(szText)
			popup.launch(true, PopupStates.POPUPSTATE_QUEUED)

	def initScriptData(self):
		
		# Set default script data manually since we need defaults for all values in the array before any functions can be called on them
		iDefaultNumNukesFired = 0
		
		aScriptData = [iDefaultNumNukesFired]
		gc.getGame().setScriptData(pickle.dumps(aScriptData))
		
	def getGameNumNukes(self):
		aszScriptData = pickle.loads(gc.getGame().getScriptData())
		iNumNukes = aszScriptData[0]		 # Num Nukes Fired is 0th element
		return iNumNukes
	def setGameNumNukes(self, iValue):
		aszScriptData = pickle.loads(gc.getGame().getScriptData())
		aszScriptData[0] = iValue
		gc.getGame().setScriptData(pickle.dumps(aszScriptData))
		
		self.checkNukeStuff(aszScriptData[0])
		
	def changeGameNumNukes(self, iChange):
		aszScriptData = pickle.loads(gc.getGame().getScriptData())
		aszScriptData[0] = aszScriptData[0] + iChange
		gc.getGame().setScriptData(pickle.dumps(aszScriptData))
		
		self.checkNukeStuff(aszScriptData[0])
		
	def checkNukeStuff(self, iNumNukes):
		
		print("iNumNukes")
		print(iNumNukes)
		print("g_iNumNukesGameOver")
		print(g_iNumNukesGameOver)
		
		# Amount of nukes matches Warning message
		if (CyMap().plot(0,0).getScriptData() == "Scenario") or (CyMap().plot(0,0).getScriptData() != "Scenario"):
			if (iNumNukes == g_iNumNukesWarningMessage):
				self.addPopup("", localText.getText("TXT_KEY_ROUND_THREE_NUKE_WARNING", ()))
		
		# Amount of nukes matches Game over
			elif (iNumNukes == g_iNumNukesGameOver):
			
			# Show the screen that shows the world 'sploding
				CvScreensInterface.showNextWarScreen()
				gc.getGame().setWinner(18, 3)